//                                                                                
//    Copyright (c) 2005/2006 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

//extern int getline (char **lineptr, size_t *n, FILE *stream);
#include <getline.h>


//windows notes:
// - no / delimiter, but backslash
// - move, instead of mv
// - directory args to move not finished with
// Windows XP:
// - commands don't work, if executable paths is surrounded by " "	
// Windows XP service pack 2:
// - move of directories from C: to other drives is forbidden
        
// restrictions:

// !! wildcarts don't allows \"tmp/*.wav\"

// lowercase file extensions for system() calls


        
#define MAX_TOKEN_PER_FILE		1024
#define MAX_FILE_SIZE			(1024*1024)
#define MAX_SCRIPT_DEPTH		5

#define MAX_AGP_FILES			50
#define MAX_CDDA_TRACKS			100
#define AGP_EXT		".AGP"

#define CMD_LEN		1000
char cmd[CMD_LEN], cwd[MAX_SCRIPT_DEPTH][CMD_LEN];

#define MAX_TXT_LINES	(1+8)




#ifdef WIN32
#    define COPY      "copy"
#    define DELETE    "del"
#    define DELIMITER "\\"
#    define MOVE      "move"
#else
#    define COPY      "cp"
#    define DELETE    "rm"
#    define DELIMITER "/"
#    define MOVE      "mv"
#endif

#define COMMAND(str, ...)  {\
    snprintf(cmd, CMD_LEN, str, __VA_ARGS__);\
    if(debugLevel >= 2)\
        printf("Command:\n\t%s\n", cmd);\
    system(cmd);\
}




char 	*aif = NULL, *agp = NULL, *atlas = NULL, *cdroot = NULL, *GAMENAME = NULL, *cel = NULL, *map = NULL, *pal = NULL, *ipi = NULL, *iso = NULL, \
        *create = NULL, *readme = NULL, *spr = NULL, *tmp = NULL, *tool = NULL, *txt = NULL;
char	*agps[MAX_AGP_FILES*10], *cdda[MAX_CDDA_TRACKS];
int		nAgps = 0, nCdda = 0, scriptDepth = 0, debugLevel = 1;


enum {TYPE_SINGLE, TYPE_MULTI, TYPE_MULTICART};


void usage()
{
    printf(
        "\n\nAtlas Creator\n"
        "by Rockin'-B, www.rockin-b.de\n\n"
        "ABOUT:\n"
        "Atlas is a multi game boot disc application that runs on the SEGA Saturn,\n"
        "allowing up to %i games to be put on a single CD and selected with a menu.\n\n"
        "An Atlas Game Pack (AGP) contains a game that can be put on a Atlas CD\n"
        "using Atlas Creator. Included is all game data, audio tracks,\n"
        "a readme and presentation data for Atlas.\n\n"
        "Atlas Creator can create ready to burn Saturn CD Images from one or more\n"
        "Atlas Game Packs. It allows you to modify it's content with UpdateIso.bat.\n"
        "You can also create Atlas CD's to be used with an Atlas Boot cartridge.\n"
        "It can generate a html website showing all specs of your AGP collection.\n"
		"Of course, Atlas Game Packs can be created, too.\n\n"
        "Atlas Creator is driven by commands, each command is made of a name\n"
        "followed by a value. Commands can be supplied on the command line and\n"
        "in script files as well. With command \"script <name>\" you can recursively\n"
        "execute scripts up to a depth of %i!\n"
		"Each script file can have up to %i pairs of <command-name> <command-value>.\n"
		"If <command-value> is specified as NULL, it's deleted.\n\n"
        "USAGE:\n"
        "\tatlscrtr <command-name> <command-value> <command-name> <command-value> ...\n\n",
        MAX_AGP_FILES, MAX_SCRIPT_DEPTH, MAX_TOKEN_PER_FILE/2
    );
#ifdef WIN32
    system("PAUSE");
#endif
    printf(
        "COMMANDS:\n"
        "create\t\t- Perform the action: multi, multicart, single, pack, atlas, website\n"
        "script\t\t- open a script file\n"
        "aif\t\t- audio file information\n"
        "agp\t\t- directory of the Atlas Game Pack or directory containing multiple AGPs\n"
        "atlas\t\t- Atlas AGP file\n"
        "cdroot\t\t- contains what should go into cd root\n"
        "GAMENAME\t- game name on CD, <= 8 uppercase characters\n"
        "cel\t\t- full screen info image\n"
        "map\t\t- full screen info image\n"
        "pal\t\t- full screen info image\n"
        "ipi\t\t- game IP.BIN file\n"
        "iso\t\t- empty directory in which to create the ISO image\n"
        "readme\t\t- readme file\n"
        "spr\t\t- screenshots thumbnail\n"
        "tmp\t\t- empty temporary directory\n"
        "tool\t\t- contains tools\n"
        "txt\t\t- short text description file\n"
        "agps\t\t- list of Atlas Game Packs\n"
        "cdda\t\t- list of CD audio files\n"
        "debug\t\t- set debug level to activate messages"
#ifdef WIN32
    );
    system("PAUSE");
    printf(
#endif
        "\n\nREQUIRED SETTINGS:\n"
        "Create Multi Game Saturn Disc (create multi/multicart):\n"
        "\trequired: agps or/and agp, atlas, tool, tmp, iso\n\toptional: ipi, readme\n"
        "\nCreate Single Game Saturn Disc (create single):\n"
        "\trequired: agps, atlas, tool, tmp, iso\n\toptional: ipi, readme\n"
        "\nCreate Atlas Game Pack (create pack):\n"
        "\trequired: tool, tmp, GAMENAME, cdroot, ipi\n\toptional: agp, txt, spr, cel+map+pal, aif, readme, cdda\n"
        "\nCreate Atlas AGP (create atlas):\n"
        "\trequired: tool, tmp, GAMENAME, cdroot, ipi\n\toptional: agp, readme, cdda\n"
        "\nCreate AGP website (create website):\n"
        "\trequired: agp, tool, tmp\n"
    );
#ifdef WIN32
    system("PAUSE");
#endif
}



char *check_extension(char *str, char *ext)
{
#if 0	// error, because path can contain . and ..
    char *ext_start = strrchr(str, '.');
    if(ext_start == NULL) {
//    printf("Here 2\n");
        return NULL;
}
        
    if(strcasecmp(ext_start, ext) == 0)
        return ext_start;
    else {
//    printf("Here 3: ext_start: %s ext: %s\n", ext_start, ext);
     return NULL;
     }
#else
	int i, l1, l2;
	l1 = strlen(str);
	l2 = strlen(ext);
	if(l1 <= l2)
		return NULL;
	for(i = 0; i < l2; i++) {
		if(tolower(*(str + l1 - 1 - i)) != tolower(*(ext + l2 - 1 - i)))
			return NULL;
	}
	return str + l1 - l2;
#endif
}

int get_prefix(char *agp, char *prefix)
{
    int i, j;
    char *ptr;
    
    if(prefix == NULL || agp == NULL)
        return 0;

    if(debugLevel >= 4)
        printf("agp: %s\n", agp);

    *prefix = '\0';

    ptr = check_extension(agp, AGP_EXT);
    if(ptr == NULL)
        return 0;
    // search back to front
    for(i = 0; (--ptr >= agp) && (i < 8) && (isalnum(*ptr) || (*ptr == '_')); i++) {
// what did I do that for? Does not allow names with _
//        if(*ptr == '_')
//            i = 0;
    }
    // copy
    for(j = 0; j < i; j++)
       *(prefix + j) = (char)toupper(*(ptr+j+1));
//    memcpy(prefix, ptr+1, i);
    *(prefix + j) = '\0';
    
    if(debugLevel >= 4)
        printf("prefix: %s\n", prefix);
    return 1;
}

int register_audio_files(FILE *cue, int *tno, char *tmp, char *iso)
{
    DIR *myDir;
    struct dirent *ep;

    if((cue == NULL) || (tno == NULL) || (tmp == NULL) || (iso == NULL))
        return 0;
    
    if(debugLevel >= 3)
        printf("raf: tmp: %s, iso: %s\n", tmp, iso);

    // look into the Atlas Game Pack directory
    if(debugLevel >= 2)
        printf("Now looking for audio files in directory: %s.\n", tmp);
    myDir = opendir(tmp);
    if(myDir == NULL) {
        fprintf(stderr, "Could not open directory: %s!\n", tmp);
        exit(-1);
    }
    
    // collect all .wav and .mp3 files
    while (ep = readdir(myDir)) {
        // valid audio file ?
#ifdef WIN32
        if((check_extension(ep->d_name, ".WAV") || check_extension(ep->d_name, ".MP3"))) {
#else
        if((ep->d_type == DT_REG) && (check_extension(ep->d_name, ".WAV") || check_extension(ep->d_name, ".MP3"))) {
#endif
            if(check_extension(ep->d_name, ".WAV"))
                fprintf(cue, "FILE \"tracks/%s\" WAVE\n\tTRACK %2i AUDIO%s\n\t\tINDEX 01 00:00:00\n", ep->d_name, *tno + 1, *tno == 1 ? "\n\t\tPREGAP 00:02:00" : "");
            else if(check_extension(ep->d_name, ".MP3"))
                fprintf(cue, "FILE \"tracks/%s\" MP3\n\tTRACK %2i AUDIO%s\n\t\tINDEX 01 00:00:00\n", ep->d_name, *tno + 1, *tno == 1 ? "\n\t\tPREGAP 00:02:00" : "");
            if(debugLevel >= 2)
                printf("Found audio file: %s\n", ep->d_name);
            *tno += 1;
        } else if(debugLevel > 1)
			fprintf(stderr, "Skipping invalid file: %s !\n", ep->d_name);
    }


    if(debugLevel >= 2)
        printf("Now closing directory: %s.\n", tmp);
    if(closedir(myDir))
        fprintf(stderr, "Error closing directory: %s!\n", tmp);
        
    return 1;
}


int insert_track_number(char *filename, int tno)
{
    char *lineptr[MAX_TXT_LINES];
    size_t linesize[MAX_TXT_LINES];
    int i;
    FILE *fp;

    if((filename == NULL) || (tno == 0))
        return 0;
        
    // insert track number in first line
    
    // open original file
    fp = fopen (filename, "r");
    if(fp == NULL) {
        fprintf(stderr, "Error open file %s!\n", filename);
        return 0;
    }
    // read lines of original file
    for(i = 0; i < MAX_TXT_LINES; i++) {
        lineptr[i] = NULL;
        linesize[i] = 0;
        getline (&(lineptr[i]), &(linesize[i]), fp);
    }
    // close original file
    if(fclose (fp))
        fprintf(stderr, "Error closing file %s!\n", filename);
    // open modified file for writing, truncated to 0
    fp = fopen (filename, "w");
    if(fp == NULL) {
        fprintf(stderr, "Error open file %s!\n", filename);
        return 0;
    }
    // write modified file
    fprintf(fp, "%i\n", tno);
    for(i = 1; i < MAX_TXT_LINES; i++) {
        if(lineptr[i] != NULL) {
            fputs(lineptr[i], fp);
            if(linesize[i] > 0)
                free(lineptr[i]);
        } else fprintf(fp, "\n");
    }
    // close original file
    if(fclose (fp))
        fprintf(stderr, "Error closing file %s!\n", filename);
    return 1;
}

void unpack_agp1(char *agp, char *tool, char *tmp)
{
    // move to tmp directory and rename
    COMMAND(MOVE " %s \"%s" DELIMITER "tmp.tar.bz2\"", agp, tmp);
    // decompress bzip2, keep the original
//        COMMAND("\"%s" DELIMITER "bzip2\" -dk \"%s" DELIMITER "tmp.tar.bz2\"", tool, tmp);
    COMMAND("%s" DELIMITER "bzip2 -dk \"%s" DELIMITER "tmp.tar.bz2\"", tool, tmp);
    // move back
    COMMAND(MOVE " \"%s" DELIMITER "tmp.tar.bz2\" %s", tmp, agp);        
    // unpack tar (audio + tar data + readme)
    chdir(tmp);
//        COMMAND("\"%s" DELIMITER "tar\" -xf \"tmp.tar\"", tool);
    COMMAND("%s" DELIMITER "tar -xf \"tmp.tar\"", tool);
    chdir(cwd[scriptDepth]);
    // remove tar
    COMMAND(DELETE " \"%s" DELIMITER "tmp.tar\"", tmp);
}

int process_agp(char *agp, char *tool, char *tmp, char *iso, FILE *cue, int *tno)
{
    char GAMENAME[8+1];

    // extract the base name
    if(get_prefix(agp, GAMENAME)) {
        if(debugLevel > 0)
            printf("\n\nNow processing file:\n%s\n", agp);
		unpack_agp1(agp, tool, tmp);
        // data files
            // move data tar
        COMMAND(MOVE " \"%s" DELIMITER "%s.TAR\" \"%s" DELIMITER "data" DELIMITER "%s.TAR\"", tmp, GAMENAME, iso, GAMENAME);
			// move readme
        COMMAND(COPY " \"%s" DELIMITER "%s.TXT\" \"%s" DELIMITER "data" DELIMITER "README" DELIMITER "\"", tmp, GAMENAME, iso);
        COMMAND(MOVE " \"%s" DELIMITER "%s.TXT\" \"%s" DELIMITER "README" DELIMITER "\"", tmp, GAMENAME, iso);
#if 1
#if 0   // doesn't work in windows
        snprintf(cmd, CMD_LEN, "\"%s" DELIMITER "data\"", iso);
        chdir(cmd);
#else
//        getcwd(cmd, CMD_LEN);
//        printf("CWD is: %s\n", cmd);
        chdir(iso);
        chdir("data");
//        COMMAND("mkdir beenhere", 0);
#endif
            // unpack data tar
//        COMMAND("\"%s" DELIMITER "tar\" -xf %s.TAR", tool, GAMENAME);
        COMMAND("%s" DELIMITER "tar -xf %s.TAR", tool, GAMENAME);
            // remove data tar
        COMMAND(DELETE " %s.TAR", GAMENAME);
        chdir(cwd[scriptDepth]);
#endif
        // audio files
            // insert tno
        snprintf(cmd, CMD_LEN, "%s" DELIMITER "data" DELIMITER "%s.TXT", iso, GAMENAME);
        if(tno != NULL)
            insert_track_number(cmd, *tno);
            // update CUE sheet
        register_audio_files(cue, tno, tmp, iso);
            // change to iso directory
        if(getcwd (cmd, CMD_LEN)) {
            if(debugLevel >= 2)
                printf("Current working directory is:\n\t%s\n", cmd);
        }

            // copy audio files
        COMMAND(COPY " %s" DELIMITER "*.WAV \"%s" DELIMITER "tracks\"", tmp, iso);
        COMMAND(COPY " %s" DELIMITER "*.MP3 \"%s" DELIMITER "tracks\"", tmp, iso);
            // remove audio files
        chdir(tmp);
        COMMAND(DELETE " *.WAV", 0);    // 0 is just a dummy to get the va macro compiled
        COMMAND(DELETE " *.MP3", 0); 
        chdir(cwd[scriptDepth]);
        return 1;
    } else { 
        fprintf(stderr, "Skipping invalid file: %s !\n", agp);
        return 0;
    }
}

// tool MUST be absolute

// atlas is the directory containing all Atlas files
int create_iso(char **agps, unsigned int nAgps, char *atlas, char *tool, char *tmp, char *iso, char *ipi, char *readme, int type)
{
    int i, tno;
    FILE *cue, *bat;
    char GAMENAME[8+1];
    //, atlas_agp[CMD_LEN];
            
    if((tool == NULL) || (tmp == NULL) || (iso == NULL) || (agps == NULL) || (nAgps < 1)) {
        fprintf(stderr,
            "Wrong arguments to create_iso!\n"
            "tool: %s\n"
            "tmp: %s\n"
            "iso: %s\n"
            "nAgps: %i\n",
            tool, tmp, iso, nAgps
        );
        return -1;
    }    
    if(debugLevel > 0)
        printf("\n\nStarting to create files:\nAtlas.cue\nAtlas.iso\n");

    // create iso and data + audio subfolders
    COMMAND("mkdir \"%s\"", iso);
    COMMAND("mkdir \"%s" DELIMITER "data\"", iso);
    COMMAND("mkdir \"%s" DELIMITER "data" DELIMITER "README\"", iso);
    COMMAND("mkdir \"%s" DELIMITER "tracks\"", iso);
    COMMAND("mkdir \"%s" DELIMITER "README\"", iso);

    // create tmp
    COMMAND("mkdir %s", tmp);

    // change to iso directory
    chdir(iso);    
    
    // init CUE sheet,
    // first audio track is for Atlas
//    snprintf(cmd, CMD_LEN, "\"%s/Atlas.cue\"", iso);
    cue = fopen ("Atlas.cue", "w");
    // return to initial directory
    chdir(cwd[scriptDepth]);
    if(cue != NULL) 
        fputs ("FILE \"tracks/Atlas.iso\" BINARY\n\tTRACK 01 MODE1/2048\n\t\tINDEX 01 00:00:00\n\t\tPOSTGAP 00:02:00\n", cue);
    else fprintf(stderr, "Error opening CUE sheet for writing!\n");
    tno = 2;
    if(type != TYPE_SINGLE) {
        // unpack the Atlas data
#if 0
        snprintf(atlas_agp, CMD_LEN, "%s" DELIMITER "ATLAS.AGP", atlas);
        process_agp(atlas_agp, tool, tmp, iso, cue, &tno);
#else
        process_agp(atlas, tool, tmp, iso, cue, &tno);
        // copy readme
        if(readme) {
	        COMMAND(COPY " \"%s\" \"%s" DELIMITER "data" DELIMITER "README" DELIMITER "\"", readme, iso);
	        COMMAND(COPY " \"%s\" \"%s" DELIMITER "README" DELIMITER "\"", readme, iso);
		}
		// move IP.BIN
		get_prefix(atlas, GAMENAME);
		COMMAND(MOVE " \"%s" DELIMITER "data" DELIMITER "%s.IPI\" \"%s" DELIMITER "IP.BIN\"", iso, GAMENAME, iso);
#endif
    } else if(nAgps > 1)
        nAgps = 1;	// to be save
    
    // process agp
    for(i = 0; i < nAgps; i++)
        process_agp(*(agps + i), tool, tmp, iso, cue, &tno);
    
    // finish CUE sheet
    if(fclose (cue))
        fprintf(stderr, "Error closing CUE sheet!\n");

    // open batch file
    	// change to iso directory
    chdir(iso);    
    bat = fopen ("UpdateIso.bat", "w");
	    // return to initial directory
    chdir(cwd[scriptDepth]);
    if(bat == NULL) 
    	fprintf(stderr, "Error open batch file for writing!\n");

    // generate ISO
    if((type == TYPE_SINGLE) && get_prefix(*(agps + 0), GAMENAME)) {
        if(ipi) {
//            COMMAND("\"%s" DELIMITER "mkisofs\" -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, ipi, iso, iso, GAMENAME);
            COMMAND("%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, ipi, iso, iso, GAMENAME);
            fprintf(bat, "%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, ipi, iso, iso, GAMENAME);
        } else {
//            COMMAND("\"%s" DELIMITER "mkisofs\" -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s" DELIMITER "data" DELIMITER "%s.IPI -full-iso9660-filenames -o %s" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, iso, GAMENAME, iso, iso, GAMENAME);
            COMMAND("%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s" DELIMITER "data" DELIMITER "%s.IPI -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, iso, GAMENAME, iso, iso, GAMENAME);
            fprintf(bat, "%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s" DELIMITER "data" DELIMITER "%s.IPI -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data" DELIMITER "%s", tool, iso, GAMENAME, iso, iso, GAMENAME);
        }
    } else if(type == TYPE_MULTICART) {
	        COMMAND("%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, iso, iso);
	        fprintf(bat, "%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, iso, iso);
	} else {
//        COMMAND("\"%s" DELIMITER "mkisofs\" -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, ipi, iso, iso);
		if(ipi) {
	        COMMAND("%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, ipi, iso, iso);
	        fprintf(bat, "%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, ipi, iso, iso);
        } else {
			COMMAND("%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s" DELIMITER "IP.BIN  -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, iso, iso, iso);
			fprintf(bat, "%s" DELIMITER "mkisofs -sysid \"SEGA SATURN\" -volid \"Atlas\" -volset \"Atlas\" -publisher \"The Rockin\'-B\" -preparer \"The Rockin\'-B\" -appid \"Atlas\" -abstract \"ABS.TXT\" -copyright \"CPY.TXT\" -biblio \"BIB.TXT\" -generic-boot %s" DELIMITER "IP.BIN  -full-iso9660-filenames -o %s" DELIMITER "tracks" DELIMITER "Atlas.iso %s" DELIMITER "data", tool, iso, iso, iso);
		}
//		leave IP.BIN to allo
//		COMMAND(DELETE " %s" DELIMITER "IP.BIN", iso);
    }
    // finish batch file
    fprintf(bat, "\nPAUSE\n");
    if(fclose(bat))
        fprintf(stderr, "Error closing batch file!\n");
    
    // delete temporary directory
    COMMAND("rmdir \"%s\"", tmp);

    if(debugLevel > 0)
        printf("\n\nFinished creating files:\n%s" DELIMITER "Atlas.cue\n%s" DELIMITER "Atlas.iso\n", iso, iso);

    return 0;
}



int read_agp_dir(char *agp, int maxAgp)
{
    DIR *myDir;
    struct dirent *ep;
    int i;
    
    // insert code here...
//    printf("\n\nHello stranger!\n\n");
//
//    if(getcwd (cmd, CMD_LEN))
//        printf("Current working directory is:\n\t%s\n", cmd);
//    else fprintf(stderr, "Error getting current working directory!\n");

    // look into the Atlas Game Pack directory
    if(debugLevel > 1)
        printf("Now looking into directory: %s\n", agp);
    myDir = opendir (agp);
    if(myDir == NULL) {
        fprintf(stderr, "Could not open directory:  %s\n", agp);
        return -1;
    }
    
    // collect paths to all Atlas Game Packs
    i = 0;
    while (ep = readdir (myDir)) {
        // valid AGP ?
#ifdef WIN32
        if(check_extension(ep->d_name, AGP_EXT)) {
#else
        if((ep->d_type == DT_REG) && check_extension(ep->d_name, AGP_EXT)) {
#endif
            if(nAgps < maxAgp) {
                if(agps[nAgps])
                    free(agps[nAgps]);
                // create a path
                agps[nAgps] = (char *)malloc(strlen(agp) + strlen(ep->d_name) + 2);\
                if(agps[nAgps]) {
                     strcpy(agps[nAgps], agp);
                     strcat(agps[nAgps], DELIMITER);
                     strcat(agps[nAgps], ep->d_name);
                } else fprintf(stderr, "Error allocating string memory!\n");
                nAgps++;
            } else {
                fprintf(stderr, "Maximum number of AGPs reached!\n");
            }
            if(debugLevel > 2)
                printf("Collected path: %s\n", ep->d_name);
            i++;
        } else if(debugLevel > 1)
			fprintf(stderr, "Skipping invalid file: %s !\n", ep->d_name);
    }


    if(debugLevel > 1)
        printf("Now closing directory: %s\n", agp);
    if(closedir(myDir))
        fprintf(stderr, "Error closing directory: %s\n", agp);

    return 0;
}



int move(char *src, char *tmp, char *GAMENAME, char *ext)
{
    if((src == NULL) || (tmp == NULL) || (GAMENAME == NULL))
        return -1;
        
    if(ext == NULL) {
        COMMAND(MOVE " \"%s\" \"%s" DELIMITER "%s\"", src, tmp, GAMENAME);
    } else {
        COMMAND(MOVE " \"%s\" \"%s" DELIMITER "%s.%s\"", src, tmp, GAMENAME, ext);
    }
    
    return 0;
}



int moveback(char *src, char *tmp, char *GAMENAME, char *ext)
{
    if((src == NULL) || (tmp == NULL) || (GAMENAME == NULL))
        return -1;
        
    if(ext == NULL) {
        COMMAND(MOVE " \"%s" DELIMITER "%s\" \"%s\"", tmp, GAMENAME, src);
    } else {
        COMMAND(MOVE " \"%s" DELIMITER "%s.%s\" \"%s\"", tmp, GAMENAME, ext, src);
    }
    
    return 0;
}



// ensure that tmp directory does not exist

// tool path must be absolute
// or relative to tmp path

// agp is the filename path
// of the generated agp file
// can be omitted, then the file will stay in tmp
int create_agp(char *tool, char *tmp, char *GAMENAME, char *cdroot, char *ipi, char *agp, char *txt, char *spr, char *cel, char *map, char *pal, char *aif, char *readme, char **cdda, int nCdda, char makeAtlas)
{
    int i;
    // check args
    if((tool == NULL) || (tmp == NULL) || (GAMENAME == NULL) || (cdroot == NULL) || (ipi == NULL)) {
        fprintf(stderr,
            "Wrong arguments to pack!\n"
            "tool: %s\n"
            "tmp: %s\n"
            "GAMENAME: %s\n"
            "cdroot: %s\n"
            "ipi: %s\n",
            tool, tmp, GAMENAME, cdroot, ipi
        );
        return -1;
    }
    if(debugLevel > 0)
        printf("\n\nStarting to create file:\n%s.AGP\n", GAMENAME);

    // create tmp
    COMMAND("mkdir %s", tmp);
    // go into tmp
    chdir(tmp);

	if(makeAtlas) {
		COMMAND(COPY " %s" DELIMITER "*.* %s", cdroot, tmp);
		COMMAND("mkdir %s" DELIMITER "README", tmp);
		COMMAND(COPY " %s" DELIMITER "README" DELIMITER "*.* %s" DELIMITER "README", cdroot, tmp);
		COMMAND("mkdir %s" DELIMITER "AUTORUN", tmp);
		COMMAND(COPY " %s" DELIMITER "AUTORUN" DELIMITER "*.* %s" DELIMITER "AUTORUN", cdroot, tmp);
	    COMMAND("%s" DELIMITER "tar --remove-files -cf \"%s.TAR\" *.* README AUTORUN", tool, GAMENAME);
	    COMMAND("rmdir %s" DELIMITER "README", tmp);
	    COMMAND("rmdir %s" DELIMITER "AUTORUN", tmp);

	    move(ipi, ".", GAMENAME, "IPI");
	    COMMAND("%s" DELIMITER "tar -rf \"%s.TAR\" %s.IPI", tool, GAMENAME, GAMENAME);
	    moveback(ipi, ".", GAMENAME, "IPI");
	} else {
	    // move to tmp and rename 
#if 0
		COMMAND("mkdir %s" DELIMITER "%s", tmp, GAMENAME);
		COMMAND(COPY " %s" DELIMITER "*.* %s" DELIMITER "%s", cdroot, tmp, GAMENAME);
//#if WIN32 // no subdirectories possible	otherwise    
////		COMMAND("%s" DELIMITER "cp --recursive %s" DELIMITER "*.* %s" DELIMITER "%s", tool, cdroot, tmp, GAMENAME);
//		COMMAND("%s" DELIMITER "mv %s" DELIMITER " %s" DELIMITER "%s", tool, cdroot, tmp, GAMENAME);
//#else
//		COMMAND(COPY " --recursive %s" DELIMITER "*.* %s" DELIMITER "%s", cdroot, tmp, GAMENAME);
//#endif
// WinXP sp2: not allowed from C:	
#else
#	ifdef WIN32
		COMMAND("%s" DELIMITER "mv %s" DELIMITER " %s" DELIMITER "%s", tool, cdroot, tmp, GAMENAME);
#	else
	    move(cdroot, ".", GAMENAME, NULL);
#	endif
#endif
	    move(ipi, ".", GAMENAME, "IPI");
	    move(txt, ".", GAMENAME, "TXT");
	    move(spr, ".", GAMENAME, "SPR");
	    move(cel, ".", GAMENAME, "CEL");
	    move(map, ".", GAMENAME, "MAP");
	    move(pal, ".", GAMENAME, "PAL");
	    move(aif, ".", GAMENAME, "AIF");
	    
	    // create TAR from files and folders in cd root
	    // and atlas customizations
//	    COMMAND("\"%s" DELIMITER "tar\" -cf \"%s.TAR\" \"%s\" \"%s.IPI\" %s%s %s%s %s%s %s%s", tool, GAMENAME, GAMENAME, GAMENAME, txt == NULL ? "" : GAMENAME, txt == NULL ? "" : ".TXT", spr == NULL ? "" : GAMENAME, spr == NULL ? "" : ".SPR", img == NULL ? "" : GAMENAME, img == NULL ? "" : ".IMG", aif == NULL ? "" : GAMENAME, aif == NULL ? "" : ".AIF");
			// first tar game's cdroot and remove the copy automatically
#if 0
	    COMMAND("%s" DELIMITER "tar --remove-files -cf \"%s.TAR\" \"%s\"", tool, GAMENAME, GAMENAME);
		COMMAND("rmdir %s" DELIMITER "%s", tmp, GAMENAME);
//#ifdef WIN32 // no subdirectories
//	    COMMAND("%s" DELIMITER "rm --recursive %s" DELIMITER "%s", tool, tmp, GAMENAME);
//#else
//	    COMMAND("rm --recursive %s" DELIMITER "%s", tmp, GAMENAME);
//#endif
			// then the rest, which has been moved here
	    COMMAND("%s" DELIMITER "tar -rf \"%s.TAR\" \"%s.IPI\" %s%s %s%s %s%s %s%s", tool, GAMENAME, GAMENAME, txt == NULL ? "" : GAMENAME, txt == NULL ? "" : ".TXT", spr == NULL ? "" : GAMENAME, spr == NULL ? "" : ".SPR", img == NULL ? "" : GAMENAME, img == NULL ? "" : ".IMG", aif == NULL ? "" : GAMENAME, aif == NULL ? "" : ".AIF");
	    // move back from tmp and rename 
#else
	    COMMAND("%s" DELIMITER "tar -cf \"%s.TAR\" \"%s\" \"%s.IPI\" %s%s %s%s %s%s %s%s %s%s %s%s", tool, GAMENAME, GAMENAME, GAMENAME, txt == NULL ? "" : GAMENAME, txt == NULL ? "" : ".TXT", spr == NULL ? "" : GAMENAME, spr == NULL ? "" : ".SPR", cel == NULL ? "" : GAMENAME, cel == NULL ? "" : ".CEL", map == NULL ? "" : GAMENAME, map == NULL ? "" : ".MAP", pal == NULL ? "" : GAMENAME, pal == NULL ? "" : ".PAL", aif == NULL ? "" : GAMENAME, aif == NULL ? "" : ".AIF");
#	ifdef WIN32
		COMMAND("%s" DELIMITER "mv %s" DELIMITER "%s %s", tool, tmp, GAMENAME, cdroot);
#	else
	    moveback(cdroot, ".", GAMENAME, NULL);
#	endif
#endif
	    moveback(ipi, ".", GAMENAME, "IPI");
	    moveback(txt, ".", GAMENAME, "TXT");
	    moveback(spr, ".", GAMENAME, "SPR");
	    moveback(cel, ".", GAMENAME, "CEL");
	    moveback(map, ".", GAMENAME, "MAP");
	    moveback(pal, ".", GAMENAME, "PAL");
	    moveback(aif, ".", GAMENAME, "AIF");
	}    
    // create AGP
        // readme
    move(readme, ".", GAMENAME, "TXT");
//    COMMAND("\"%s" DELIMITER "tar\" -cf \"%s.AGP\" \"%s.TAR\" %s%s", tool, GAMENAME, GAMENAME, readme == NULL ? "" : GAMENAME, readme == NULL ? "" : ".TXT");
    COMMAND("%s" DELIMITER "tar -cf \"%s.AGP\" \"%s.TAR\" %s%s", tool, GAMENAME, GAMENAME, readme == NULL ? "" : GAMENAME, readme == NULL ? "" : ".TXT");
    moveback(readme, ".", GAMENAME, "TXT");
        // audion tracks
    for(i = 0; i < nCdda; i++) {
        if(*(cdda + i) != NULL) {
            if(check_extension(*(cdda + i), ".WAV")) {
                // move audio track
                COMMAND(MOVE " \"%s\" \"%s" DELIMITER "%s_%i.WAV\"", *(cdda + i), ".", GAMENAME, i+2);
                // add to tar
//                COMMAND("\"%s" DELIMITER "tar\" -rf \"%s.AGP\" \"%s_%i.WAV\"", tool, GAMENAME, GAMENAME, i+2);
                COMMAND("%s" DELIMITER "tar -rf \"%s.AGP\" \"%s_%i.WAV\"", tool, GAMENAME, GAMENAME, i+2);
                // move back
                COMMAND(MOVE " \"%s" DELIMITER "%s_%i.WAV\" \"%s\"", ".", GAMENAME, i+2, *(cdda + i));
            } else if(check_extension(*(cdda + i), ".MP3")) {
                COMMAND(MOVE " \"%s\" \"%s" DELIMITER "%s_%i.MP3\"", *(cdda + i), ".", GAMENAME, i+2);
//                COMMAND("\"%s" DELIMITER "tar\" -rf \"%s.AGP\" \"%s_%i.MP3\"", tool, GAMENAME, GAMENAME, i+2);
                COMMAND("%s" DELIMITER "tar -rf \"%s.AGP\" \"%s_%i.MP3\"", tool, GAMENAME, GAMENAME, i+2);
                COMMAND(MOVE " \"%s" DELIMITER "%s_%i.MP3\" \"%s\"", ".", GAMENAME, i+2, *(cdda + i));
            } else {
                fprintf(stderr, "Wrong cd audio file extension: %s\n", *(cdda + i));
            }
        }
    }
        // bzip2
//    COMMAND("\"%s" DELIMITER "bzip2\" -z9 \"%s.AGP\"\n", tool, GAMENAME);
    COMMAND("%s" DELIMITER "bzip2 -z9 \"%s.AGP\"\n", tool, GAMENAME);
        // rename
    COMMAND(MOVE " \"%s.AGP.bz2\" \"%s.AGP\"", GAMENAME, GAMENAME);
    
    // remove temporary TAR file
    COMMAND(DELETE " \"%s" DELIMITER "%s.TAR\"", ".", GAMENAME);
    
    // apply final name
    if(agp != NULL) {
        COMMAND(MOVE " \"%s" DELIMITER "%s.AGP\" \"%s" DELIMITER "%s.AGP\"", ".", GAMENAME, agp, GAMENAME);
//        COMMAND(MOVE " \"%s" DELIMITER "%s.AGP\" \"%s\"\nrmdir \"%s\"", ".", GAMENAME, agp, ".");
    }
    // return to initial directory
    chdir(cwd[scriptDepth]);

    // delete temporary directory
    COMMAND("rmdir \"%s\"", tmp);
    
    // finished
    if(debugLevel > 0) {
        if(agp)
            printf("\n\nFinished creating file:\n%s" DELIMITER "%s.AGP\n", agp, GAMENAME);
        else printf("\n\nFinished creating file:\n%s" DELIMITER "%s.AGP\n", tmp, GAMENAME);
    }
    return 0;
}

#include <sys/stat.h>
#include <time.h>


//int my_selector(const struct dirent *dir)
//{
//#ifdef WIN32
//	if(check_extension(dir->d_name, AGP_EXT))
//#else
//	if((dir->d_type == DT_REG) && check_extension(dir->d_name, AGP_EXT))
//#endif
//		return 1;
//	else return 0;
//}
//
//int my_cmp(const void *a, const void *b)
//{
//	
//}

#if 1
int create_website(char *tool, char *tmp, char *agp)
{
    DIR *tmpDir;
    struct dirent *tmpEp;
    struct stat my_stat;
    FILE *out, *in;
    int i, j;
    char GAMENAME[8+1], description[MAX_TXT_LINES * (8 + 2) + 100], CDDA, TXT, SPR, IMG, AIF, README;
    
    if((tool == NULL) || (tmp == NULL) || (agp == NULL)) {
        fprintf(stderr,
            "Wrong arguments to pack!\n"
            "tool: %s\n"
            "tmp: %s\n"
            "agp: %s\n",
            tool, tmp, agp
        );
        return -1;
    }
    // create tmp
    COMMAND("mkdir %s", tmp);
    
    chdir(agp);
    out = fopen ("my-agp-collection.html", "w");
    if(!out) 
    	fprintf(stderr, "Error opening HTML file for writing!\n");
    else {
		fprintf(out,
			"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN>\n"
			"<html>\n"
			"<head>\n"
			"\t<meta content=\"text/html; charset=ISO-8859-1\"http-equiv=\"content-type\">\n"
			"\t<title>My AGP collection</title>\n"
			"</head>\n"
			"<body>\n"
			"<div style=\"text-align: center;\">\n"
			"<h1>My AGP Collection</h1>\n"
			"</div>\n"
			"<br>\n"
			"<br>\n"
			"<table style=\"width: 100%%; text-align: left;\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\">\n"
			"<tbody>\n"
			"<tr>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 15%%;\"><big>Thumbnail</big></td>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 15%%;\"><big>Name / Size</big></td>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 30%%;\"><big>Description</big></td>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 15%%;\"><big>Date</big></td>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 10%%;\"><big>Audio Tracks</big></td>\n"
			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 15%%;\"><big>Content</big></td>\n"
			"</tr>\n"
		);
	}

    
    // look into the Atlas Game Pack directory
    // collect paths to all Atlas Game Packs
	read_agp_dir(agp, MAX_AGP_FILES*10);

    for(j = 0; j < nAgps; j++) {
//		char tmpstr[CMD_LEN+1];
		// collect info about this AGP
		TXT = SPR = IMG = AIF = README = CDDA = 0;
		description[0] = '\0';
		GAMENAME[0] = '\0';
		if(!get_prefix(agps[j], GAMENAME))
			fprintf(stderr, "Error getting GAMENAME prefix of file: %s!\n", agps[j]);
		if(stat(agps[j], &my_stat))
			fprintf(stderr, "Error getting attributes of file: %s!\n", agps[j]);
		// unpack AGP
//		snprintf(tmpstr, CMD_LEN, "%s" DELIMITER "%s", agp, ep->d_name);
		unpack_agp1(agps[j], tool, tmp);
		// look for CDDA and README
		
	    if(debugLevel > 1)
	        printf("Now looking into directory: %s\n", tmp);
	    tmpDir = opendir (tmp);
	    if(tmpDir == NULL) {
	        fprintf(stderr, "Could not open directory:  %s\n", tmp);
	        return -1;
	    }
	    while (tmpEp = readdir (tmpDir)) {
			if(check_extension(tmpEp->d_name, ".WAV") || check_extension(tmpEp->d_name, ".MP3"))
				CDDA++;
			else if(check_extension(tmpEp->d_name, ".TXT"))
				README = 1;
		}
	    if(debugLevel > 1)
	        printf("Now closing directory: %s\n", tmp);
	    if(closedir(tmpDir))
    		fprintf(stderr, "Error closing directory: %s\n", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.WAV", tmp);
	    COMMAND(DELETE " %s" DELIMITER "*.MP3", tmp); 
	    COMMAND(DELETE " %s" DELIMITER "*.TXT", tmp);
	    // look for TXT, SPR, IMG, AIF
	    chdir(tmp);
        COMMAND("%s" DELIMITER "tar -xf %s.TAR", tool, GAMENAME);
        chdir(agp);
            // remove data tar
        COMMAND(DELETE " %s" DELIMITER "%s.TAR", tmp, GAMENAME);
	    
	    if(debugLevel > 1)
	        printf("Now looking into directory: %s\n", tmp);
	    tmpDir = opendir (tmp);
	    if(tmpDir == NULL) {
	        fprintf(stderr, "Could not open directory:  %s\n", tmp);
	        return -1;
	    }
	    while (tmpEp = readdir (tmpDir)) {
//			printf("HiHo, I see file: %s\n", tmpEp->d_name);
			if(check_extension(tmpEp->d_name, ".SPR"))
				SPR = 1;
			else if(check_extension(tmpEp->d_name, ".AIF"))
				AIF = 1;
			else if(check_extension(tmpEp->d_name, ".CEL") || check_extension(tmpEp->d_name, ".MAP") || check_extension(tmpEp->d_name, ".PAL"))
				IMG = 1;
			else if(check_extension(tmpEp->d_name, ".TXT")) {
				char c;
				TXT = 1;
	    	    chdir(tmp);
			    in = fopen (tmpEp->d_name, "r");
			    if(in == NULL) {
			        fprintf(stderr, "Error open file %s!\n", tmpEp->d_name);
			        return 0;
			    }
			    // skip tno
			    do {
					c = fgetc(in);
				} while(isdigit(c) || c != '\n');
				// read description
				i = 0;
			    do {
					c = fgetc(in);
					if(isprint(c))
						description[i++] = c;
					else if(c == '\n') {
						description[i++] = ' ';
//						description[i++] = '<';
//						description[i++] = 'b';
//						description[i++] = 'r';
//						description[i++] = '>';
					}
				} while(c != EOF && i < sizeof(description));
				description[i] = '\0';
			    // close original file
			    if(fclose (in))
			        fprintf(stderr, "Error closing file %s!\n", tmpEp->d_name);
		        chdir(agp);
			}
		}
	    if(debugLevel > 1)
	        printf("Now closing directory: %s\n", tmp);
	    if(closedir(tmpDir))
    		fprintf(stderr, "Error closing directory: %s\n", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.IPI", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.TXT", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.SPR", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.CEL", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.MAP", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.PAL", tmp);
        COMMAND(DELETE " %s" DELIMITER "*.AIF", tmp);
#ifdef WIN32 // no subdirectories
    	COMMAND("%s" DELIMITER "rm --recursive %s" DELIMITER "%s", tool, tmp, GAMENAME);
#else
    	COMMAND("rm --recursive %s" DELIMITER "%s", tmp, GAMENAME);
#endif
		fprintf(out,
		    "<tr>\n"
			"<td style=\"vertical-align: top; width: 15%%;\"><a href=\"%s.JPG\"><img alt=\"\" src=\"%s_THUMB.JPG\"style=\"border: 0px solid ;\"></a></td>\n"
			"<td style=\"vertical-align: top; width: 15%%;\"><a href=\"%s.AGP\">%s.AGP</a><br>%.3f MB</td>\n"
			"<td style=\"vertical-align: top; width: 30%%;\">%s</td>\n"
			"<td style=\"vertical-align: top; width: 15%%;\">%s</td>\n"
			"<td style=\"vertical-align: top; width: 10%%;\">%i</td>\n"
			"<td style=\"vertical-align: top; width: 15%%;\">%s %s %s %s %s</td>\n"
			"</tr>\n",
//			GAMENAME, GAMENAME, ep->d_name, ep->d_name, description, (float)my_stat.st_size / (1024.0 * 1024.0), ctime(&my_stat.st_mtime), CDDA, TXT ? "TXT " : "", SPR ? "SPR " : "", IMG ? "IMG " : "", AIF ? "AIF " : "", README ? "README " : ""
			GAMENAME, GAMENAME, GAMENAME, GAMENAME, (float)my_stat.st_size / (1024.0 * 1024.0), description, ctime(&my_stat.st_mtime), CDDA, TXT ? "TXT " : "", SPR ? "SPR " : "", IMG ? "IMG " : "", AIF ? "AIF " : "", README ? "README " : ""
		);
    }


    fprintf(out,
		"</tbody>\n"
		"</table>\n"
		"</body>\n"
		"</html>\n"
	);
    if(fclose(out))
        fprintf(stderr, "Error closing HTML file!\n");
	chdir(cwd[scriptDepth]);

    // delete temporary directory
    COMMAND("rmdir \"%s\"", tmp);

    // finished
    if(debugLevel > 0) {
		printf("\n\nFinished creating file:\n%s" DELIMITER "my-agp-collection.html\n", agp);
    }

    return 0;
}

#else
//int create_website(char *tool, char *tmp, char *agp)
//{
//    DIR *myDir, *tmpDir;
//    struct dirent *ep, *tmpEp;
//    off_t offset;	// cannot nest readdir()
//    struct stat my_stat;
//    FILE *out, *in;
//    int i;
//    char GAMENAME[8+1], description[MAX_TXT_LINES * 22], CDDA, TXT, SPR, IMG, AIF, README;
//    
//    if((tool == NULL) || (tmp == NULL) || (agp == NULL)) {
//        fprintf(stderr,
//            "Wrong arguments to pack!\n"
//            "tool: %s\n"
//            "tmp: %s\n"
//            "agp: %s\n",
//            tool, tmp, agp
//        );
//        return -1;
//    }
//    // create tmp
//    COMMAND("mkdir %s", tmp);
//    
//    chdir(agp);
//    out = fopen ("my-agp-collection.html", "w");
//    if(!out) 
//    	fprintf(stderr, "Error opening HTML file for writing!\n");
//    else {
//		fprintf(out,
//			"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN>\n"
//			"<html>\n"
//			"<head>\n"
//			"\t<meta content=\"text/html; charset=ISO-8859-1\"http-equiv=\"content-type\">\n"
//			"\t<title>My AGP collection</title>\n"
//			"</head>\n"
//			"<body>\n"
//			"<div style=\"text-align: center;\">\n"
//			"<h1>My AGP Collection</h1>\n"
//			"</div>\n"
//			"<br>\n"
//			"<br>\n"
//			"<table style=\"width: 100%%; text-align: left;\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\">\n"
//			"<tbody>\n"
//			"<tr>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 20%%;\"><big>Thumbnail</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 10%%;\"><big>Name</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 30%%;\"><big>Description</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 10%%;\"><big>Size</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 10%%;\"><big>Date</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 10%%;\"><big>Audio Tracks</big></td>\n"
//			"<td style=\"vertical-align: top; font-weight: bold; text-align: center; width: 25%%;\"><big>Content</big></td>\n"
//			"</tr>\n"
//		);
//	}
//
//    
//    // look into the Atlas Game Pack directory
//    if(debugLevel > 1)
//        printf("Now looking into directory: %s\n", agp);
//    myDir = opendir (agp);
//    if(myDir == NULL) {
//        fprintf(stderr, "Could not open directory:  %s\n", agp);
//        return -1;
//    }
//    
//    // collect paths to all Atlas Game Packs
//    while (ep = readdir (myDir)) {
//        // valid AGP ?
//#ifdef WIN32
//        if(check_extension(ep->d_name, AGP_EXT)) {
//#else
//        if((ep->d_type == DT_REG) && check_extension(ep->d_name, AGP_EXT)) {
//#endif
//			char tmpstr[CMD_LEN+1];
//			// collect info about this AGP
//			TXT = SPR = IMG = AIF = README = CDDA = 0;
//			description[0] = '\0';
//			GAMENAME[0] = '\0';
//    		if(!get_prefix(ep->d_name, GAMENAME))
//				fprintf(stderr, "Error getting GAMENAME prefix of file: %s!\n", ep->d_name);
//			if(stat(ep->d_name, &my_stat))
//				fprintf(stderr, "Error getting attributes of file: %s!\n", ep->d_name);
//			// unpack AGP
//			snprintf(tmpstr, CMD_LEN, "%s" DELIMITER "%s", agp, ep->d_name);
//			unpack_agp1(tmpstr, tool, tmp);
//			// look for CDDA and README
//				// save dir
//			offset = telldir(myDir);
//				
//		    if(debugLevel > 1)
//		        printf("Now looking into directory: %s\n", tmp);
//		    tmpDir = opendir (tmp);
//		    if(tmpDir == NULL) {
//		        fprintf(stderr, "Could not open directory:  %s\n", tmp);
//		        return -1;
//		    }
//		    while (tmpEp = readdir (tmpDir)) {
//				if(check_extension(tmpEp->d_name, ".WAV") || check_extension(tmpEp->d_name, ".MP3"))
//					CDDA++;
//				else if(check_extension(tmpEp->d_name, ".TXT"))
//					README = 1;
//			}
//		    if(debugLevel > 1)
//		        printf("Now closing directory: %s\n", tmp);
//		    if(closedir(tmpDir))
//        		fprintf(stderr, "Error closing directory: %s\n", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.WAV", tmp);
//    	    COMMAND(DELETE " %s" DELIMITER "*.MP3", tmp); 
//    	    COMMAND(DELETE " %s" DELIMITER "*.TXT", tmp);
//    	    // look for TXT, SPR, IMG, AIF
//    	    chdir(tmp);
//	        COMMAND("%s" DELIMITER "tar -xf %s.TAR", tool, GAMENAME);
//	        chdir(agp);
//	            // remove data tar
//	        COMMAND(DELETE " %s" DELIMITER "%s.TAR", tmp, GAMENAME);
//    	    
//		    if(debugLevel > 1)
//		        printf("Now looking into directory: %s\n", tmp);
//		    tmpDir = opendir (tmp);
//		    if(tmpDir == NULL) {
//		        fprintf(stderr, "Could not open directory:  %s\n", tmp);
//		        return -1;
//		    }
//		    while (tmpEp = readdir (tmpDir)) {
//				printf("HiHo, I see file: %s\n", tmpEp->d_name);
//				if(check_extension(tmpEp->d_name, ".SPR"))
//					SPR = 1;
//				else if(check_extension(tmpEp->d_name, ".AIF"))
//					AIF = 1;
//				else if(check_extension(tmpEp->d_name, ".CEL") || check_extension(tmpEp->d_name, ".MAP") || check_extension(tmpEp->d_name, ".PAL"))
//					IMG = 1;
//				else if(check_extension(tmpEp->d_name, ".TXT")) {
//					char c;
//					TXT = 1;
//		    	    chdir(tmp);
//				    in = fopen (tmpEp->d_name, "r");
//				    if(in == NULL) {
//				        fprintf(stderr, "Error open file %s!\n", tmpEp->d_name);
//				        return 0;
//				    }
//				    // skip tno
//				    do {
//						c = fgetc(in);
//					} while(isdigit(c) || c != '\n');
//					// read description
//					i = 0;
//				    do {
//						c = fgetc(in);
//						if(isalnum(c) || c == ' ')
//							description[i++] = c;
//					} while(c != EOF && i < sizeof(description));
//					description[i] = '\0';
//				    // close original file
//				    if(fclose (in))
//				        fprintf(stderr, "Error closing file %s!\n", tmpEp->d_name);
//			        chdir(agp);
//				}
//			}
//		    if(debugLevel > 1)
//		        printf("Now closing directory: %s\n", tmp);
//		    if(closedir(tmpDir))
//        		fprintf(stderr, "Error closing directory: %s\n", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.IPI", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.TXT", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.SPR", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.CEL", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.MAP", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.PAL", tmp);
//	        COMMAND(DELETE " %s" DELIMITER "*.AIF", tmp);
//#ifdef WIN32 // no subdirectories
//	    	COMMAND("%s" DELIMITER "rm --recursive %s" DELIMITER "%s", tool, tmp, GAMENAME);
//#else
//	    	COMMAND("rm --recursive %s" DELIMITER "%s", tmp, GAMENAME);
//#endif
//				// restoring directory
////		    myDir = opendir (agp);
////		    if(myDir == NULL) {
////		        fprintf(stderr, "Could not open directory:  %s\n", agp);
////		        return -1;
////		    }
//		    seekdir(myDir, offset);
//			ep = readdir (myDir);
//
//			fprintf(out,
//			    "<tr>\n"
//				"<td style=\"vertical-align: top; width: 20%%;\"><a href=\"%s.jpg\"><img alt=\"\" src=\"%s_thumb.jpg\"style=\"border: 0px solid ;\"></a></td>\n"
//				"<td style=\"vertical-align: top; width: 10%%;\"><a href=\"%s\">%s</a></td>\n"
//				"<td style=\"vertical-align: top; width: 30%%;\">%s</td>\n"
//				"<td style=\"vertical-align: top; width: 10%%;\">%f MB</td>\n"
//				"<td style=\"vertical-align: top; width: 10%%;\">%s</td>\n"
//				"<td style=\"vertical-align: top; width: 10%%;\">%i</td>\n"
//				"<td style=\"vertical-align: top; width: 25%%;\">%s %s %s %s %s</td>\n"
//				"</tr>\n",
//				GAMENAME, GAMENAME, ep->d_name, ep->d_name, description, (float)my_stat.st_size / (1024.0 * 1024.0), ctime(&my_stat.st_mtime), CDDA, TXT ? "TXT " : "", SPR ? "SPR " : "", IMG ? "IMG " : "", AIF ? "AIF " : "", README ? "README " : ""
//			);
//				
//        } else fprintf(stderr, "Skipping invalid file: %s !\n", ep->d_name);
//    }
//
//
//    if(debugLevel > 1)
//        printf("Now closing directory: %s\n", agp);
//    if(closedir(myDir))
//        fprintf(stderr, "Error closing directory: %s\n", agp);
//
//    fprintf(out,
//		"</tbody>\n"
//		"</table>\n"
//		"</body>\n"
//		"</html>\n"
//	);
//    if(fclose(out))
//        fprintf(stderr, "Error closing HTML file!\n");
//	chdir(cwd[scriptDepth]);
//
//    // delete temporary directory
//    COMMAND("rmdir \"%s\"", tmp);
//
//    // finished
//    if(debugLevel > 0) {
//		printf("\n\nFinished creating file:\n%s" DELIMITER "my-agp-collection.html\n", agp);
//    }
//
//    return 0;
//}
#endif



#ifdef WIN32
#define CORRECT_PATH(dst) {\
        int i2;\
        for(i2 = 0; dst[i2]; i2++) {\
            if(dst[i2] == '/')\
                dst[i2] = '\\';\
        }\
        if(dst[i2-1] == '\\')\
            dst[i2-1] = '\0';\
}
#else
#define CORRECT_PATH(dst) {\
        int i2;\
        for(i2 = 0; dst[i2]; i2++) {\
            if(dst[i2] == '\\')\
                dst[i2] = '/';\
        }\
        if(dst[i2-1] == '/')\
            dst[i2-1] = '\0';\
}
#endif

#define ASSIGN_STRING(dst) {\
    if(dst)\
        free(dst);\
    if(strcasecmp(argv[++i], "NULL") == 0) {\
		dst = NULL;\
	} else {\
	    if(debugLevel > 2)\
	        printf("correcting path? %s\n", argv[i]);\
	    if(argv[i][0] == '.') {\
	        if(debugLevel > 2)\
	            printf("correcting path\n");\
	        dst = (char *)malloc(strlen(cwd[scriptDepth]) + strlen(argv[i]) + 2);\
	        if(dst) {\
	             strcpy(dst, cwd[scriptDepth]);\
	             strcat(dst, DELIMITER);\
	             strcat(dst, argv[i]);\
	        }\
	    } else dst = strdup(argv[i]);\
	    if(!dst)\
	        fprintf(stderr, "Error allocating string memory!\n");\
	    else {\
	        CORRECT_PATH(dst);\
	    }\
	}\
}

int read_file(char *filename);

void read_args(int argc, char *argv[])
{
    int i;
    // reset arg variables
//    aif = agp = atlas = cdroot = GAMENAME = img = ipi = iso = create = readme = spr = tmp = tool = txt = NULL;
//    nAgps = nCdda = 0;

    // for filename resolution
    if(!getcwd(cwd[scriptDepth], CMD_LEN) && debugLevel > 2)
        fprintf(stderr, "Failed getting current working directory!\n");

    for (i = 0; (i+1) < argc; i++) {
        if(strcmp(argv[i], "aif") == 0) {
            ASSIGN_STRING(aif);
        } else if(strcmp(argv[i], "agps") == 0) {
			// delete?
			if(strcasecmp(argv[i+1], "NULL") == 0) {
				int k;
				for(k = 0; k < nAgps; k++) {
					if(agps[k])
						free(agps[k]);
				}
				nAgps = 0;
				i++;
			} else {
	            if(nAgps < MAX_AGP_FILES) {
	                ASSIGN_STRING(agps[nAgps]);
	                nAgps++;
	            } else {
	                i++;
	                fprintf(stderr, "Maximum number of AGPs reached!\n");
	            }
			}
        } else if(strcmp(argv[i], "agp") == 0) {
            ASSIGN_STRING(agp);
        } else if(strcmp(argv[i], "atlas") == 0) {
            ASSIGN_STRING(atlas);
        } else if(strcmp(argv[i], "cdda") == 0) {
			// delete?
			if(strcasecmp(argv[i+1], "NULL") == 0) {
				int k;
				for(k = 0; k < nCdda; k++) {
					if(cdda[k])
						free(cdda[k]);
				}
				nCdda = 0;
				i++;
			} else {
	            if(nCdda < MAX_CDDA_TRACKS) {
	                ASSIGN_STRING(cdda[nCdda]);
	                nCdda++;
	            } else {
	                i++;
	                fprintf(stderr, "Maximum number of CDDAs reached!\n");
	            }
			}
        } else if(strcmp(argv[i], "cdroot") == 0) {
            ASSIGN_STRING(cdroot);
        } else if(strcmp(argv[i], "GAMENAME") == 0) {
            ASSIGN_STRING(GAMENAME);
        } else if(strcmp(argv[i], "cel") == 0) {
            ASSIGN_STRING(cel);
        } else if(strcmp(argv[i], "map") == 0) {
            ASSIGN_STRING(map);
        } else if(strcmp(argv[i], "pal") == 0) {
            ASSIGN_STRING(pal);
        } else if(strcmp(argv[i], "ipi") == 0) {
            ASSIGN_STRING(ipi);
        } else if(strcmp(argv[i], "iso") == 0) {
            ASSIGN_STRING(iso);
        } else if(strcmp(argv[i], "readme") == 0) {
            ASSIGN_STRING(readme);
        } else if(strcmp(argv[i], "spr") == 0) {
            ASSIGN_STRING(spr);
        } else if(strcmp(argv[i], "tmp") == 0) {
            ASSIGN_STRING(tmp);
        } else if(strcmp(argv[i], "tool") == 0) {
            ASSIGN_STRING(tool);
        } else if(strcmp(argv[i], "txt") == 0) {
            ASSIGN_STRING(txt);
        } else if(strcmp(argv[i], "create") == 0) {
            ASSIGN_STRING(create);
            if(strcmp(create, "single") == 0) 
                create_iso(agps, nAgps, atlas, tool, tmp, iso, ipi, readme, TYPE_SINGLE);
            else if(strcmp(create, "multicart") == 0) 
                create_iso(agps, nAgps, atlas, tool, tmp, iso, ipi, readme, TYPE_MULTICART);
            else if(strcmp(create, "multi") == 0) {
                if(agp)
                    read_agp_dir(agp, MAX_AGP_FILES);
                create_iso(agps, nAgps, atlas, tool, tmp, iso, ipi, readme, TYPE_MULTI);
            } else if(strcmp(create, "pack") == 0) {
                create_agp(tool, tmp, GAMENAME, cdroot, ipi, agp, txt, spr, cel, map, pal, aif, readme, cdda, nCdda, 0);
            } else if(strcmp(create, "atlas") == 0) {
                create_agp(tool, tmp, GAMENAME, cdroot, ipi, agp, txt, spr, cel, map, pal, aif, readme, cdda, nCdda, 1);
            } else if(strcmp(create, "website") == 0) {
                create_website(tool, tmp, agp);
            } else fprintf(stderr, "Wrong create: %s\n", create);
            nAgps = nCdda = 0;
        } else if(strcmp(argv[i], "debug") == 0) {
            debugLevel = atoi(argv[++i]);
        } else if(strcmp(argv[i], "script") == 0) {
            if(scriptDepth < MAX_SCRIPT_DEPTH) {
                scriptDepth++;
                read_file(argv[++i]);
                scriptDepth--;
            } else fprintf(stderr, "Warning: cannot process script %s, maximum script recursion is %i!\n", argv[i-1], MAX_SCRIPT_DEPTH);
        } else fprintf(stderr, "Skipping wrong command: %s\n", argv[i]);
    }
}


int read_file(char *filename)
{
    int argc;
    char *argv[MAX_TOKEN_PER_FILE], *filebuffer, *str;
    FILE *in;
    size_t size;
    
    filebuffer = (char *)malloc(MAX_FILE_SIZE+1);
    if(!filebuffer) {
        fprintf(stderr, "Out of memory, can't read file %s!\n", filename);
        return -1;
    }
    // read tokens
    if(debugLevel >= 2)
        printf("Opening command file: %s.\n", filename);
    in = fopen(filename, "r");
    if(!in) {
        fprintf(stderr, "Cannot open file: %s!\n", filename);
        return -1;
    }
#if 0
    for(argc = 0; argc < MAX_TOKEN_PER_FILE; argc++) {
        char *variable;
        printf("reading arg %i..\n", argc);
        argv[argc] = NULL;
        if (scanf ("%a[a-zA-Z0-9/\\:.]", &(argv[argc])) != 1)
            break;
    }
    // free memory
    for(i = 1; i < argc; i++) {
        if(argv[i])
            free(argv[i]);
    }
#else
    size = fread(filebuffer, 1, MAX_FILE_SIZE, in);
    if(size == MAX_FILE_SIZE)
        fprintf(stderr, "Warning: file %s may exceed maximum allowed size %i!\n", filename, MAX_FILE_SIZE);
    filebuffer[size] = '\0';
    for(argc = 0, str = strtok(filebuffer, " \n\t\r"); str; argc++, str = strtok (NULL, " \n\t\r") ) {
		if(argc >= MAX_TOKEN_PER_FILE)
			fprintf(stderr, "WARNING: the maximum tokens per file are reached: %i!\nThe rest of the file will be skipped!\nLast token read was: %s!\n", MAX_TOKEN_PER_FILE, argv[MAX_TOKEN_PER_FILE-1]);
        argv[argc] = str;
    	if(debugLevel >= 3)
        	printf("Found token: %s\n", str);
    }
#endif
    fclose(in);
    if(debugLevel >= 3)
        printf("Found %i tokens.\n", argc);
    // perform action
    // change to script directory
    {
        char *str = (char *)strdup(filename);
        if(!str)
            fprintf(stderr, "Error, out of memory!\n");
        else {
            // extract directory path
            int o;
            for(o = strlen(str); o > 0; o--) {
                if(str[o] == '/' || str[o] == '\\') {
                    str[o] = '\0';
                    break;
                }
            }
            if(debugLevel > 2)
                printf("Changing into script directory: %s\n", str);
            chdir(str);
            free(str);
        }
    }
    read_args(argc, argv);
    chdir(cwd[scriptDepth]);
    free(filebuffer);
    if(debugLevel >= 2)
        printf("Finished command file: %s.\n", filename);
    return 0;
}




// avoid the warning in Dev-Cpp
//int main (int argc, const char *argv[])
int main (int argc, char *argv[])
{    
//	printf("Testing...\n");
//	create_website(".", ".", "D:/dev/SATURN/SGL/PROJECTS/AtlasCreator/examples/agp");
//	return 0;

//	system("\"D:\\dev\\SATURN\\SGL\\PROJECTS\\AtlasCreator\\Windows\\tools\\tar\" -cf \"MYAGP.TAR\" \"MYAGP\" \"MYAGP.IPI\" \"MYAGP.TXT\" \"MYAGP.AIF\"");
//	system("D:\\dev\\SATURN\\SGL\\PROJECTS\\AtlasCreator\\Windows\\tools\\tar -cf \"MYAGP.TAR\"");
//	system("pause");
//    char *my_argv[5];
//    my_argv[1] = "debug";
//    my_argv[2] = "4";
//    argc += 2;
    if(argc < 2 || !(argc & 1)) {
        usage();
        return -1;
    }
    // get cwd
    if(!getcwd(cwd[0], CMD_LEN) && debugLevel > 2)
        fprintf(stderr, "Failed getting current working directory!\n");

    read_file("atlscrtr.ini");
    read_args(argc-1, argv+1);
    
    if(debugLevel >= 1)
        printf("Finished!\n");
    // necessary?
    chdir(cwd[0]);
    return 0;
}

