#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@" 

if {![info exists vTcl(sourcing)]} {

    # Provoke name search
    catch {package require bogus-package-name}
    set packageNames [package names]

    package require BWidget
    switch $tcl_platform(platform) {
	windows {
	}
	default {
	    option add *ScrolledWindow.size 14
	}
    }
    
    package require Tk
    switch $tcl_platform(platform) {
	windows {
            option add *Button.padY 0
	}
	default {
            option add *Scrollbar.width 10
            option add *Scrollbar.highlightThickness 0
            option add *Scrollbar.elementBorderWidth 2
            option add *Scrollbar.borderWidth 2
	}
    }
    
}

#############################################################################
# Visual Tcl v1.60 Project
#


#################################
# VTCL LIBRARY PROCEDURES
#

if {![info exists vTcl(sourcing)]} {
#############################################################################
## Library Procedure:  Window

proc ::Window {args} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    global vTcl
    foreach {cmd name newname} [lrange $args 0 2] {}
    set rest    [lrange $args 3 end]
    if {$name == "" || $cmd == ""} { return }
    if {$newname == ""} { set newname $name }
    if {$name == "."} { wm withdraw $name; return }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists} {
                wm deiconify $newname
            } elseif {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[winfo exists $newname] && [wm state $newname] == "normal"} {
                vTcl:FireEvent $newname <<Show>>
            }
        }
        hide    {
            if {$exists} {
                wm withdraw $newname
                vTcl:FireEvent $newname <<Hide>>
                return}
        }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}
#############################################################################
## Library Procedure:  vTcl:DefineAlias

proc ::vTcl:DefineAlias {target alias widgetProc top_or_alias cmdalias} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    global widget
    set widget($alias) $target
    set widget(rev,$target) $alias
    if {$cmdalias} {
        interp alias {} $alias {} $widgetProc $target
    }
    if {$top_or_alias != ""} {
        set widget($top_or_alias,$alias) $target
        if {$cmdalias} {
            interp alias {} $top_or_alias.$alias {} $widgetProc $target
        }
    }
}
#############################################################################
## Library Procedure:  vTcl:DoCmdOption

proc ::vTcl:DoCmdOption {target cmd} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    ## menus are considered toplevel windows
    set parent $target
    while {[winfo class $parent] == "Menu"} {
        set parent [winfo parent $parent]
    }

    regsub -all {\%widget} $cmd $target cmd
    regsub -all {\%top} $cmd [winfo toplevel $parent] cmd

    uplevel #0 [list eval $cmd]
}
#############################################################################
## Library Procedure:  vTcl:FireEvent

proc ::vTcl:FireEvent {target event {params {}}} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    ## The window may have disappeared
    if {![winfo exists $target]} return
    ## Process each binding tag, looking for the event
    foreach bindtag [bindtags $target] {
        set tag_events [bind $bindtag]
        set stop_processing 0
        foreach tag_event $tag_events {
            if {$tag_event == $event} {
                set bind_code [bind $bindtag $tag_event]
                foreach rep "\{%W $target\} $params" {
                    regsub -all [lindex $rep 0] $bind_code [lindex $rep 1] bind_code
                }
                set result [catch {uplevel #0 $bind_code} errortext]
                if {$result == 3} {
                    ## break exception, stop processing
                    set stop_processing 1
                } elseif {$result != 0} {
                    bgerror $errortext
                }
                break
            }
        }
        if {$stop_processing} {break}
    }
}
#############################################################################
## Library Procedure:  vTcl:Toplevel:WidgetProc

proc ::vTcl:Toplevel:WidgetProc {w args} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    if {[llength $args] == 0} {
        ## If no arguments, returns the path the alias points to
        return $w
    }
    set command [lindex $args 0]
    set args [lrange $args 1 end]
    switch -- [string tolower $command] {
        "setvar" {
            foreach {varname value} $args {}
            if {$value == ""} {
                return [set ::${w}::${varname}]
            } else {
                return [set ::${w}::${varname} $value]
            }
        }
        "hide" - "show" {
            Window [string tolower $command] $w
        }
        "showmodal" {
            ## modal dialog ends when window is destroyed
            Window show $w; raise $w
            grab $w; tkwait window $w; grab release $w
        }
        "startmodal" {
            ## ends when endmodal called
            Window show $w; raise $w
            set ::${w}::_modal 1
            grab $w; tkwait variable ::${w}::_modal; grab release $w
        }
        "endmodal" {
            ## ends modal dialog started with startmodal, argument is var name
            set ::${w}::_modal 0
            Window hide $w
        }
        default {
            uplevel $w $command $args
        }
    }
}
#############################################################################
## Library Procedure:  vTcl:WidgetProc

proc ::vTcl:WidgetProc {w args} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    if {[llength $args] == 0} {
        ## If no arguments, returns the path the alias points to
        return $w
    }

    set command [lindex $args 0]
    set args [lrange $args 1 end]
    uplevel $w $command $args
}
#############################################################################
## Library Procedure:  vTcl:toplevel

proc ::vTcl:toplevel {args} {
    ## This procedure may be used free of restrictions.
    ##    Exception added by Christian Gavin on 08/08/02.
    ## Other packages and widget toolkits have different licensing requirements.
    ##    Please read their license agreements for details.

    uplevel #0 eval toplevel $args
    set target [lindex $args 0]
    namespace eval ::$target {set _modal 0}
}
}


if {[info exists vTcl(sourcing)]} {

proc vTcl:project:info {} {
    set base .top47
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 0
        set runvisible 1
    }
    namespace eval ::widgets::$base.lab56 {
        array set save {-justify 1 -text 1 -wraplength 1}
    }
    namespace eval ::widgets::$base.but57 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$base.but58 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$base.but59 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$base.but60 {
        array set save {-command 1 -text 1}
    }
    set base .top61
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 0
        set runvisible 1
    }
    namespace eval ::widgets::$base.lab74 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_3_0 $base.lab74
    namespace eval ::widgets::$site_3_0.but85 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_3_0.ent86 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$base.lab75 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_3_0 $base.lab75
    namespace eval ::widgets::$site_3_0.lis87 {
        array set save {-background 1 -listvariable 1}
    }
    namespace eval ::widgets::$site_3_0.but88 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_3_0.but89 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.but77 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$base.lab47 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_3_0 $base.lab47
    namespace eval ::widgets::$site_3_0.lab49 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_4_0 $site_3_0.lab49
    namespace eval ::widgets::$site_4_0.but53 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_4_0.ent54 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$site_3_0.lab52 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_4_0 $site_3_0.lab52
    namespace eval ::widgets::$site_4_0.but55 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_4_0.ent56 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$base.cpd48 {
        array set save {-text 1}
    }
    set base .top62
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 1
        set runvisible 1
    }
    namespace eval ::widgets::$base.lab80 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    namespace eval ::widgets::$base.lab81 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_3_0 $base.lab81
    namespace eval ::widgets::$site_3_0.but91 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_3_0.ent92 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$base.lab82 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_3_0 $base.lab82
    namespace eval ::widgets::$site_3_0.but94 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_3_0.ent96 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$base.but83 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.but84 {
        array set save {-command 1 -text 1}
    }
    set base .top63
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 0
        set runvisible 1
    }
    namespace eval ::widgets::$base.but57 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$base.not56 {
        array set save {-height 1 -width 1}
        namespace eval subOptions {
            array set save {-activebackground 1 -activeforeground 1 -background 1 -disabledforeground 1 -foreground 1 -text 1}
        }
    }
    set site_4_0 [$base.not56 getframe page1]
    namespace eval ::widgets::$site_4_0 {
        array set save {-background 1 -highlightbackground 1 -highlightcolor 1 -relief 1}
    }
    set site_4_0 $site_4_0
    namespace eval ::widgets::$site_4_0.cpd59 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd59
    namespace eval ::widgets::$site_5_0.ent59 {
        array set save {-_tooltip 1 -background 1 -insertbackground 1 -relief 1 -textvariable 1}
    }
    namespace eval ::widgets::$site_4_0.cpd60 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd60
    namespace eval ::widgets::$site_5_0.ent63 {
        array set save {-_tooltip 1 -background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$site_5_0.but64 {
        array set save {-_tooltip 1 -command 1 -text 1}
    }
    namespace eval ::widgets::$site_4_0.cpd61 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd61
    namespace eval ::widgets::$site_5_0.ent65 {
        array set save {-background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$site_5_0.but66 {
        array set save {-command 1 -text 1}
    }
    set site_4_1 [$base.not56 getframe page2]
    namespace eval ::widgets::$site_4_1 {
        array set save {-background 1 -highlightbackground 1 -highlightcolor 1}
    }
    set site_4_0 $site_4_1
    namespace eval ::widgets::$site_4_0.cpd63 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd63
    namespace eval ::widgets::$site_5_0.but97 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_5_0.ent57 {
        array set save {-background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$site_4_0.cpd64 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd64
    namespace eval ::widgets::$site_5_0.but104 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_5_0.ent105 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$site_5_0.fra48 {
        array set save {-borderwidth 1 -height 1 -relief 1 -width 1}
    }
    set site_6_0 $site_5_0.fra48
    namespace eval ::widgets::$site_6_0.lab49 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_6_0.cpd50 {
        array set save {-background 1 -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$site_5_0.fra51 {
        array set save {-borderwidth 1 -height 1 -relief 1 -width 1}
    }
    set site_6_0 $site_5_0.fra51
    namespace eval ::widgets::$site_6_0.lab52 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$site_6_0.cpd53 {
        array set save {-background 1 -height 1 -insertbackground 1 -width 1}
    }
    namespace eval ::widgets::$site_4_0.cpd65 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd65
    namespace eval ::widgets::$site_5_0.but100 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_5_0.ent106 {
        array set save {-background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$site_4_0.cpd66 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd66
    namespace eval ::widgets::$site_5_0.but101 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_5_0.ent107 {
        array set save {-background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$site_4_0.cpd67 {
        array set save {-background 1 -foreground 1 -highlightbackground 1 -highlightcolor 1 -text 1}
    }
    set site_5_0 $site_4_0.cpd67
    namespace eval ::widgets::$site_5_0.tex108 {
        array set save {-background 1 -height 1 -insertbackground 1 -width 1}
    }
    namespace eval ::widgets::$site_5_0.but109 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets::$site_5_0.ent110 {
        array set save {-background 1 -insertbackground 1 -textvariable 1 -width 1}
    }
    namespace eval ::widgets::$base.cpd62 {
        array set save {-command 1 -text 1}
    }
    namespace eval ::widgets_bindings {
        set tagslist {_TopLevel _vTclBalloon}
    }
    namespace eval ::vTcl::modules::main {
        set procs {
            init
            main
            selectDir
            selectFile
            pPack_read_txt
            pPack_read_readme
            pPack_create
        }
        set compounds {
        }
        set projectType single
    }
}
}

#################################
# USER DEFINED PROCEDURES
#
#############################################################################
## Procedure:  main

proc ::main {argc argv} {
#global ::widgets::.top63::cd
#set ::widgets::.top63::cd "/"
}
#############################################################################
## Procedure:  selectDir

proc ::selectDir {w s} {
global widget path
if {[string length $s] == 0} {
    set initialdir $path
} else {
    set initialdir $s
}
set dir [tk_chooseDirectory -initialdir $initialdir -parent $w -mustexist 1]
if {[string length $dir] != 0} {
    return $dir
} else {
    return $s
}
}
#############################################################################
## Procedure:  selectFile

proc ::selectFile {p s t} {
global widget path
if {[string length $s] == 0} {
    set initialfile $path
} else {
    set initialfile $s
}

set file [tk_getOpenFile -initialfile $initialfile -parent $p -filetypes $t]
if {[string length $file] != 0} {
    return $file
} else {
    return $s
}
}
#############################################################################
## Procedure:  pPack_read_txt

proc ::pPack_read_txt {s} {
global widget vPack_tno vPack_txt_height
set fin [open $s r]
# read the track number
if { [gets $fin line] != -1 } {
    set vPack_tno $line
}   
# read the lines
# delete old text
$widget(wPack_txt) delete 1.0 [expr 1.0 + $vPack_txt_height]
puts "reading description"
for { set i 0; set index 1.0 } { $i < $vPack_txt_height } { incr i;  set index [expr $index + 1.0] } {
    if { [gets $fin line] == -1 } {
        break
    }
    puts "I've read at index $index: $line"
    $widget(wPack_txt) insert $index "$line\n" 
}
close $fin
}
#############################################################################
## Procedure:  pPack_read_readme

proc ::pPack_read_readme {s} {
global widget vPack_tno vPack_txt_height
set fin [open $s r]
# read the lines
# delete old text
$widget(wPack_readme) delete 1.0 200.0
puts "reading readme"
for { set i 0; set index 1.0 } { $i < $vPack_txt_height } { incr i;  set index [expr $index + 1.0] } {
    if { [gets $fin line] == -1 } {
        break
    }
    puts "I've read at index $index: $line"
    $widget(wPack_readme) insert $index "$line\n" 
}
close $fin
}
#############################################################################
## Procedure:  pPack_create

proc ::pPack_create {} {
# An Atlas Game Pack
# got a name of up to 8 characters like GAMENAME.AGP
# It is a BZip2 compressed TAR file,
# which contains
# - all CD audio files as .wav or .mp3
# - a GAMENAME.TAR file containing all files and folders that go into CD root
# - a GAMENAME.TXT readme file
global widget vPack_name vPack_cd
set tmp_pack "/Users/thomasfuchs/Desktop/Rockin'-B Allstars/TOOL/AGP/tmp_pack"
# check required args
set strlen [string length $vPack_name]
if { [expr $strlen <= 0] || [expr $strlen > 8]} {
    puts "Wrong AGP base name: $vPack_name!"
    return
}
if { [string length $vPack_cd] == 0 } {
    puts "Wrong cd root directory: $vPack_cd!"
    return    
}
# create temporary directory
exec mkdir $tmp_pack
#exec mkdir ./tmp_pack
# create GAMENAME.TAR
#     rename directory
exec "mv \"$vPack_cd\" \"$tmp_pack/$vPack_name\""
# create the command line
set com "tar -cf $tmp_pack/$vPack_name.tar $vPack_cd"
exec $com
}

#############################################################################
## Initialization Procedure:  init

proc ::init {argc argv} {
global gamename cd ipi spr agp
global img aif txt readme 
global tno vPack_txt_height vPack_txt_width path
#lappend auto_path $env(HOME)/Desktop/Rockin'-B Allstars/TOOL/tcltk/vtcl-1.6.0/lib
set gamename "MYATLAS"
set cd ""
set ipi ""
set spr ""
set img ""
set aif ""
set txt ""
set agp ""
set readme ""
set vPack_txt_height 8
set vPack_txt_width 20
set tno 2
set path "/"
}

init $argc $argv

#################################
# VTCL GENERATED GUI PROCEDURES
#

proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    ###################
    # CREATING WIDGETS
    ###################
    wm focusmodel $top passive
    wm geometry $top 200x200+25+65; update
    wm maxsize $top 1009 738
    wm minsize $top 1 1
    wm overrideredirect $top 0
    wm resizable $top 1 1
    wm withdraw $top
    wm title $top "Wish Shell"
    bindtags $top "$top {Wish shell} all"
    vTcl:FireEvent $top <<Create>>
    wm protocol $top WM_DELETE_WINDOW "vTcl:FireEvent $top <<DeleteWindow>>"

    ###################
    # SETTING GEOMETRY
    ###################

    vTcl:FireEvent $base <<Ready>>
}

proc vTclWindow.top47 {base} {
    if {$base == ""} {
        set base .top47
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    set top $base
    ###################
    # CREATING WIDGETS
    ###################
    vTcl:toplevel $top -class Toplevel \
        -background #d9d9d9 -highlightbackground #d9d9d9 \
        -highlightcolor black 
    wm focusmodel $top passive
    wm geometry $top +34+213; update
    wm maxsize $top 1009 738
    wm minsize $top 1 1
    wm overrideredirect $top 0
    wm resizable $top 1 1
    wm deiconify $top
    wm title $top "Atlas Creator"
    vTcl:DefineAlias "$top" "wSelect" vTcl:Toplevel:WidgetProc "" 1
    bindtags $top "$top Toplevel all _TopLevel"
    vTcl:FireEvent $top <<Create>>
    wm protocol $top WM_DELETE_WINDOW "vTcl:FireEvent $top <<DeleteWindow>>"

    label $top.lab56 \
        -justify left \
        -text {Atlas:
Atlas is a multi game boot disc application for SEGA Saturn. With Atlas, you can put up to 50 games on one Saturn disc.

Atlas Game Packs:
In Atlas, each game can present additional information like screenshots, control info, short text description or even audio information. All this content is included in a single file called Atlas Game Pack.

Atlas Creator:
This application, the Atlas Maker, allows to do both, to create Saturn discs from up to 50 Atlas Game Packs as well as to create Atlas Game Packs.} \
        -wraplength 5i 
    vTcl:DefineAlias "$top.lab56" "Label2" vTcl:WidgetProc "wSelect" 1
    button $top.but57 \
        -command {Window show $widget(wMulti)} \
        -text {Create Multi Game Saturn Disc} 
    vTcl:DefineAlias "$top.but57" "Button1" vTcl:WidgetProc "wSelect" 1
    button $top.but58 \
        -command {Window show $widget(wSingle)} \
        -text {Create Single Game Saturn Disc} 
    vTcl:DefineAlias "$top.but58" "Button2" vTcl:WidgetProc "wSelect" 1
    button $top.but59 \
        -command {Window show $widget(wPack)} -text {Create Atlas Game Pack} 
    vTcl:DefineAlias "$top.but59" "Button3" vTcl:WidgetProc "wSelect" 1
    button $top.but60 \
        \
        -command {catch {destroy $widget(wMulti)};
catch {destroy $widget(wSingle)};
catch {destroy $widget(wPack)};
catch {destroy $widget(wSelect)};} \
        -text {Close All} 
    vTcl:DefineAlias "$top.but60" "Button4" vTcl:WidgetProc "wSelect" 1
    ###################
    # SETTING GEOMETRY
    ###################
    pack $top.lab56 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but57 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but58 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but59 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but60 \
        -in $top -anchor center -expand 0 -fill none -padx 5 -pady 5 \
        -side bottom 

    vTcl:FireEvent $base <<Ready>>
}

proc vTclWindow.top61 {base} {
    if {$base == ""} {
        set base .top61
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    set top $base
    ###################
    # CREATING WIDGETS
    ###################
    vTcl:toplevel $top -class Toplevel \
        -background #d9d9d9 -highlightbackground #d9d9d9 \
        -highlightcolor black 
    wm withdraw $top
    wm focusmodel $top passive
    wm geometry $top +330+64; update
    wm maxsize $top 1009 738
    wm minsize $top 1 1
    wm overrideredirect $top 0
    wm resizable $top 1 1
    wm title $top "Create Multi Game Saturn Disc"
    vTcl:DefineAlias "$top" "wMulti" vTcl:Toplevel:WidgetProc "" 1
    bindtags $top "$top Toplevel all _TopLevel"
    vTcl:FireEvent $top <<Create>>
    wm protocol $top WM_DELETE_WINDOW "vTcl:FireEvent $top <<DeleteWindow>>"

    labelframe $top.lab74 \
        -foreground black -text Name -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab74" "Labelframe1" vTcl:WidgetProc "wMulti" 1
    set site_3_0 $top.lab74
    button $site_3_0.but85 \
        -text {Select Name} 
    vTcl:DefineAlias "$site_3_0.but85" "Button5" vTcl:WidgetProc "wMulti" 1
    entry $site_3_0.ent86 \
        -background white -insertbackground black \
        -textvariable "$top\::ent86" 
    vTcl:DefineAlias "$site_3_0.ent86" "Entry1" vTcl:WidgetProc "wMulti" 1
    pack $site_3_0.but85 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side left 
    pack $site_3_0.ent86 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side left 
    labelframe $top.lab75 \
        -foreground black -text {Atlas Game Packs} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab75" "Labelframe2" vTcl:WidgetProc "wMulti" 1
    set site_3_0 $top.lab75
    listbox $site_3_0.lis87 \
        -background white -listvariable "$top\::lis87" 
    vTcl:DefineAlias "$site_3_0.lis87" "Listbox1" vTcl:WidgetProc "wMulti" 1
    button $site_3_0.but88 \
        -text {Add AGP} 
    vTcl:DefineAlias "$site_3_0.but88" "Button6" vTcl:WidgetProc "wMulti" 1
    button $site_3_0.but89 \
        -text {Remove AGP} 
    vTcl:DefineAlias "$site_3_0.but89" "Button7" vTcl:WidgetProc "wMulti" 1
    pack $site_3_0.lis87 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side left 
    pack $site_3_0.but88 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side top 
    pack $site_3_0.but89 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side bottom 
    button $top.but77 \
        -command {Window hide $widget(wMulti)} -text {Close Window} 
    vTcl:DefineAlias "$top.but77" "Button2" vTcl:WidgetProc "wMulti" 1
    labelframe $top.lab47 \
        -foreground black -text Customize -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab47" "Labelframe3" vTcl:WidgetProc "wMulti" 1
    set site_3_0 $top.lab47
    labelframe $site_3_0.lab49 \
        -foreground black -text {Main screen} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_3_0.lab49" "Labelframe4" vTcl:WidgetProc "wMulti" 1
    set site_4_0 $site_3_0.lab49
    button $site_4_0.but53 \
        -text {Select Image} 
    vTcl:DefineAlias "$site_4_0.but53" "Button3" vTcl:WidgetProc "wMulti" 1
    entry $site_4_0.ent54 \
        -background white -insertbackground black \
        -textvariable "$top\::ent54" 
    vTcl:DefineAlias "$site_4_0.ent54" "Entry2" vTcl:WidgetProc "wMulti" 1
    pack $site_4_0.but53 \
        -in $site_4_0 -anchor center -expand 0 -fill none -side left 
    pack $site_4_0.ent54 \
        -in $site_4_0 -anchor center -expand 0 -fill none -side right 
    labelframe $site_3_0.lab52 \
        -foreground black -text {Credits screen} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_3_0.lab52" "Labelframe5" vTcl:WidgetProc "wMulti" 1
    set site_4_0 $site_3_0.lab52
    button $site_4_0.but55 \
        -text {Select Image} 
    vTcl:DefineAlias "$site_4_0.but55" "Button4" vTcl:WidgetProc "wMulti" 1
    entry $site_4_0.ent56 \
        -background white -insertbackground black \
        -textvariable "$top\::ent56" 
    vTcl:DefineAlias "$site_4_0.ent56" "Entry3" vTcl:WidgetProc "wMulti" 1
    pack $site_4_0.but55 \
        -in $site_4_0 -anchor center -expand 0 -fill none -side left 
    pack $site_4_0.ent56 \
        -in $site_4_0 -anchor center -expand 0 -fill none -side right 
    pack $site_3_0.lab49 \
        -in $site_3_0 -anchor center -expand 0 -fill x -padx 5 -pady 5 \
        -side top 
    pack $site_3_0.lab52 \
        -in $site_3_0 -anchor center -expand 0 -fill x -padx 5 -pady 5 \
        -side top 
    button $top.cpd48 \
        -text {Create Multi Game Saturn Disc} 
    vTcl:DefineAlias "$top.cpd48" "Button1" vTcl:WidgetProc "wMulti" 1
    ###################
    # SETTING GEOMETRY
    ###################
    pack $top.lab74 \
        -in $top -anchor center -expand 0 -fill x -ipadx 5 -ipady 5 -padx 5 \
        -pady 5 -side top 
    pack $top.lab75 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but77 \
        -in $top -anchor center -expand 0 -fill none -padx 5 -pady 5 \
        -side bottom 
    pack $top.lab47 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.cpd48 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 

    vTcl:FireEvent $base <<Ready>>
}

proc vTclWindow.top62 {base} {
    if {$base == ""} {
        set base .top62
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    set top $base
    ###################
    # CREATING WIDGETS
    ###################
    vTcl:toplevel $top -class Toplevel \
        -background #d9d9d9 -highlightbackground #d9d9d9 \
        -highlightcolor black 
    wm withdraw $top
    wm focusmodel $top passive
    wm geometry $top 497x348+1+394; update
    wm maxsize $top 1009 738
    wm minsize $top 1 1
    wm overrideredirect $top 0
    wm resizable $top 1 1
    wm title $top "Create Single Game Saturn Disc"
    vTcl:DefineAlias "$top" "wSingle" vTcl:Toplevel:WidgetProc "" 1
    bindtags $top "$top Toplevel all _TopLevel"
    vTcl:FireEvent $top <<Create>>
    wm protocol $top WM_DELETE_WINDOW "vTcl:FireEvent $top <<DeleteWindow>>"

    labelframe $top.lab80 \
        -foreground black -text Label -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab80" "Labelframe1" vTcl:WidgetProc "wSingle" 1
    labelframe $top.lab81 \
        -foreground black -text Name -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab81" "Labelframe2" vTcl:WidgetProc "wSingle" 1
    set site_3_0 $top.lab81
    button $site_3_0.but91 \
        -text {Select Name} 
    vTcl:DefineAlias "$site_3_0.but91" "Button3" vTcl:WidgetProc "wSingle" 1
    entry $site_3_0.ent92 \
        -background white -insertbackground black -textvariable gamename 
    vTcl:DefineAlias "$site_3_0.ent92" "Entry1" vTcl:WidgetProc "wSingle" 1
    pack $site_3_0.but91 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side left 
    pack $site_3_0.ent92 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side right 
    labelframe $top.lab82 \
        -foreground black -text {Atlas Game Pack} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$top.lab82" "Labelframe3" vTcl:WidgetProc "wSingle" 1
    set site_3_0 $top.lab82
    button $site_3_0.but94 \
        \
        -command set\ agp\ \[selectFile\ \$widget(wPack)\ \$img\ \{\\\n\ \ \ \ \{\{Atlas\ Game\ Pack\}\ \ \ \ \ \ \ \{.agp\}\ \ \ \ \ \ \ \}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\] \
        -text {Select AGP} 
    vTcl:DefineAlias "$site_3_0.but94" "Button4" vTcl:WidgetProc "wSingle" 1
    entry $site_3_0.ent96 \
        -background white -insertbackground black -textvariable agp 
    vTcl:DefineAlias "$site_3_0.ent96" "Entry2" vTcl:WidgetProc "wSingle" 1
    pack $site_3_0.but94 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side left 
    pack $site_3_0.ent96 \
        -in $site_3_0 -anchor center -expand 0 -fill none -side right 
    button $top.but83 \
        -text {Create Single Game Saturn Disc} 
    vTcl:DefineAlias "$top.but83" "Button1" vTcl:WidgetProc "wSingle" 1
    button $top.but84 \
        -command {Window hide $widget(wSingle)} -text {Close Window} 
    vTcl:DefineAlias "$top.but84" "Button2" vTcl:WidgetProc "wSingle" 1
    ###################
    # SETTING GEOMETRY
    ###################
    pack $top.lab80 \
        -in $top -anchor center -expand 0 -fill none -side top 
    pack $top.lab81 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.lab82 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but83 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 
    pack $top.but84 \
        -in $top -anchor center -expand 0 -fill none -padx 5 -pady 5 \
        -side bottom 

    vTcl:FireEvent $base <<Ready>>
}

proc vTclWindow.top63 {base} {
    if {$base == ""} {
        set base .top63
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    set top $base
    ###################
    # CREATING WIDGETS
    ###################
    vTcl:toplevel $top -class Toplevel \
        -background #d9d9d9 -highlightbackground #d9d9d9 \
        -highlightcolor black 
    wm withdraw $top
    wm focusmodel $top passive
    wm geometry $top +396+22; update
    wm maxsize $top 1009 738
    wm minsize $top 1 1
    wm overrideredirect $top 0
    wm resizable $top 1 1
    wm title $top "Create Atlas Game Pack"
    vTcl:DefineAlias "$top" "wPack" vTcl:Toplevel:WidgetProc "" 1
    bindtags $top "$top Toplevel all _TopLevel"
    vTcl:FireEvent $top <<Create>>
    wm protocol $top WM_DELETE_WINDOW "vTcl:FireEvent $top <<DeleteWindow>>"

    button $top.but57 \
        -command {Window hide $widget(wPack)} -text {Close Window} 
    vTcl:DefineAlias "$top.but57" "Button1" vTcl:WidgetProc "wPack" 1
    NoteBook $top.not56 \
        -height 407 -width 375 
    vTcl:DefineAlias "$top.not56" "NoteBook1" vTcl:WidgetProc "wPack" 1
    $top.not56 insert end page1 \
        -activebackground #f9f9f9 -activeforeground black -background #d9d9d9 \
        -disabledforeground #a3a3a3 -foreground black -text Required 
    $top.not56 insert end page2 \
        -activebackground #f9f9f9 -activeforeground black -background #d9d9d9 \
        -disabledforeground #a3a3a3 -foreground black -text Optional 
    set site_4_0 [$top.not56 getframe page1]
    labelframe $site_4_0.cpd59 \
        -foreground black -text Name -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_0.cpd59" "Labelframe3" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_0.cpd59
    entry $site_5_0.ent59 \
        -background white -insertbackground black -relief flat \
        -textvariable gamename 
    vTcl:DefineAlias "$site_5_0.ent59" "Entry1" vTcl:WidgetProc "wPack" 1
    bindtags $site_5_0.ent59 "$site_5_0.ent59 Entry $top all _vTclBalloon"
    bind $site_5_0.ent59 <<SetBalloon>> {
        set ::vTcl::balloon::%W {This name is the prefix used for all Atlas specific files and folders on the Saturn disc  as well inside the Atlas Game Pack. You are restricted to use  up to 8 upper case characters. }
    }
    pack $site_5_0.ent59 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side top 
    labelframe $site_4_0.cpd60 \
        -foreground black -text {CD content root directory} \
        -background #d9d9d9 -highlightbackground #d9d9d9 \
        -highlightcolor black 
    vTcl:DefineAlias "$site_4_0.cpd60" "Labelframe4" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_0.cpd60
    entry $site_5_0.ent63 \
        -background white -insertbackground black -textvariable cd -width 40 
    vTcl:DefineAlias "$site_5_0.ent63" "Entry2" vTcl:WidgetProc "wPack" 1
    bindtags $site_5_0.ent63 "$site_5_0.ent63 Entry $top all _vTclBalloon"
    bind $site_5_0.ent63 <<SetBalloon>> {
        set ::vTcl::balloon::%W {This should point to a folder which includes all files going into  the cd root directory of your Saturn application. It must contain the binary executable.}
    }
    button $site_5_0.but64 \
        -command {set cd [selectDir $widget(wPack) ${cd}]} \
        -text {Select directory} 
    vTcl:DefineAlias "$site_5_0.but64" "Button2" vTcl:WidgetProc "wPack" 1
    bindtags $site_5_0.but64 "$site_5_0.but64 Button $top all _vTclBalloon"
    bind $site_5_0.but64 <<SetBalloon>> {
        set ::vTcl::balloon::%W {This should point to a folder which includes all files going into  the cd root directory of your Saturn application. It must contain the binary executable.}
    }
    pack $site_5_0.ent63 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    pack $site_5_0.but64 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    labelframe $site_4_0.cpd61 \
        -foreground black -text IP.BIN -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_0.cpd61" "Labelframe5" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_0.cpd61
    entry $site_5_0.ent65 \
        -background white -insertbackground black -textvariable ipi -width 40 
    vTcl:DefineAlias "$site_5_0.ent65" "Entry3" vTcl:WidgetProc "wPack" 1
    button $site_5_0.but66 \
        \
        -command set\ ipi\ \[selectFile\ \$widget(wPack)\ \$ipi\ \{\\\n\ \ \ \ \{\{IP\ file\}\ \ \ \ \ \ \ \{.bin\}\ \{.ipi\}\ \ \ \ \ \ \ \}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\] \
        -text {Select IP.BIN} 
    vTcl:DefineAlias "$site_5_0.but66" "Button3" vTcl:WidgetProc "wPack" 1
    pack $site_5_0.ent65 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    pack $site_5_0.but66 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_4_0.cpd59 \
        -in $site_4_0 -anchor center -expand 0 -fill x -padx 5 -pady 5 \
        -side top 
    pack $site_4_0.cpd60 \
        -in $site_4_0 -anchor center -expand 0 -fill x -padx 5 -pady 5 \
        -side top 
    pack $site_4_0.cpd61 \
        -in $site_4_0 -anchor center -expand 0 -fill x -padx 5 -pady 5 \
        -side top 
    set site_4_1 [$top.not56 getframe page2]
    labelframe $site_4_1.cpd63 \
        -foreground black -text Thumbnail -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_1.cpd63" "Labelframe6" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_1.cpd63
    button $site_5_0.but97 \
        \
        -command set\ spr\ \[selectFile\ \$widget(wPack)\ \$spr\ \{\\\n\ \ \ \ \{\{SPR\ file\}\ \ \ \ \ \ \ \{.bin\}\ \{.spr\}\ \ \ \ \ \ \ \}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\] \
        -text {Select Thumbnail} 
    vTcl:DefineAlias "$site_5_0.but97" "Button5" vTcl:WidgetProc "wPack" 1
    entry $site_5_0.ent57 \
        -background white -insertbackground black -textvariable spr -width 40 
    vTcl:DefineAlias "$site_5_0.ent57" "Entry8" vTcl:WidgetProc "wPack" 1
    pack $site_5_0.but97 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.ent57 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    labelframe $site_4_1.cpd64 \
        -foreground black -text Description -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_1.cpd64" "Labelframe7" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_1.cpd64
    button $site_5_0.but104 \
        \
        -command set\ txt\ \[selectFile\ \$widget(wPack)\ \$txt\ \{\\\n\ \ \ \ \{\{Text\ file\}\ \ \ \ \ \ \ \{.txt\}\ \ \ \ \ \ \ TEXT\}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\]\npPack_read_txt\ \$vPack_txt \
        -text {Load from file} 
    vTcl:DefineAlias "$site_5_0.but104" "Button6" vTcl:WidgetProc "wPack" 1
    entry $site_5_0.ent105 \
        -background white -insertbackground black -textvariable txt 
    vTcl:DefineAlias "$site_5_0.ent105" "Entry4" vTcl:WidgetProc "wPack" 1
    frame $site_5_0.fra48 \
        -borderwidth 2 -relief groove -height 122 -width 207 
    vTcl:DefineAlias "$site_5_0.fra48" "Frame1" vTcl:WidgetProc "wPack" 1
    set site_6_0 $site_5_0.fra48
    label $site_6_0.lab49 \
        -text {First line} 
    vTcl:DefineAlias "$site_6_0.lab49" "Label1" vTcl:WidgetProc "wPack" 1
    entry $site_6_0.cpd50 \
        -background white -insertbackground black -textvariable tno 
    vTcl:DefineAlias "$site_6_0.cpd50" "Entry9" vTcl:WidgetProc "wPack" 1
    pack $site_6_0.lab49 \
        -in $site_6_0 -anchor center -expand 0 -fill none -side left 
    pack $site_6_0.cpd50 \
        -in $site_6_0 -anchor center -expand 0 -fill none -side right 
    frame $site_5_0.fra51 \
        -borderwidth 2 -relief groove -height 75 -width 125 
    vTcl:DefineAlias "$site_5_0.fra51" "Frame2" vTcl:WidgetProc "wPack" 1
    set site_6_0 $site_5_0.fra51
    label $site_6_0.lab52 \
        -text Description 
    vTcl:DefineAlias "$site_6_0.lab52" "Label2" vTcl:WidgetProc "wPack" 1
    text $site_6_0.cpd53 \
        -background white -height 8 -insertbackground black -width 21 
    vTcl:DefineAlias "$site_6_0.cpd53" "wPack_txt" vTcl:WidgetProc "wPack" 1
    pack $site_6_0.lab52 \
        -in $site_6_0 -anchor center -expand 0 -fill none -side left 
    pack $site_6_0.cpd53 \
        -in $site_6_0 -anchor center -expand 0 -fill none -side right 
    pack $site_5_0.but104 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.ent105 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.fra48 \
        -in $site_5_0 -anchor center -expand 0 -fill x -ipadx 5 -ipady 5 \
        -side top 
    pack $site_5_0.fra51 \
        -in $site_5_0 -anchor center -expand 0 -fill x -ipadx 5 -ipady 5 \
        -side top 
    labelframe $site_4_1.cpd65 \
        -foreground black -text {Info screen} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_1.cpd65" "Labelframe8" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_1.cpd65
    button $site_5_0.but100 \
        \
        -command set\ img\ \[selectFile\ \$widget(wPack)\ \$img\ \{\\\n\ \ \ \ \{\{Image\ file\}\ \ \ \ \ \ \ \{.bin\}\ \{.img\}\ \ \ \ \ \ \ \}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\] \
        -text Select 
    vTcl:DefineAlias "$site_5_0.but100" "Button7" vTcl:WidgetProc "wPack" 1
    entry $site_5_0.ent106 \
        -background white -insertbackground black -textvariable img -width 40 
    vTcl:DefineAlias "$site_5_0.ent106" "Entry5" vTcl:WidgetProc "wPack" 1
    pack $site_5_0.but100 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.ent106 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    labelframe $site_4_1.cpd66 \
        -foreground black -text {Audio file} -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_1.cpd66" "Labelframe9" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_1.cpd66
    button $site_5_0.but101 \
        \
        -command set\ aif\ \[selectFile\ \$widget(wPack)\ \$aif\ \{\\\n\ \ \ \ \{\{Audio\ file\}\ \ \ \ \ \ \ \{.aif\}\ \ \ \ \ \ \ \ AIFF\}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\}\] \
        -text {Select AIF} 
    vTcl:DefineAlias "$site_5_0.but101" "Button8" vTcl:WidgetProc "wPack" 1
    entry $site_5_0.ent107 \
        -background white -insertbackground black -textvariable aif -width 40 
    vTcl:DefineAlias "$site_5_0.ent107" "Entry6" vTcl:WidgetProc "wPack" 1
    pack $site_5_0.but101 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.ent107 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    labelframe $site_4_1.cpd67 \
        -foreground black -text Readme -background #d9d9d9 \
        -highlightbackground #d9d9d9 -highlightcolor black 
    vTcl:DefineAlias "$site_4_1.cpd67" "Labelframe10" vTcl:WidgetProc "wPack" 1
    set site_5_0 $site_4_1.cpd67
    text $site_5_0.tex108 \
        -background white -height 5 -insertbackground black -width 30 
    vTcl:DefineAlias "$site_5_0.tex108" "wPack_readme" vTcl:WidgetProc "wPack" 1
    button $site_5_0.but109 \
        \
        -command set\ readme\ \[selectFile\ \$widget(wPack)\ \$readme\ \{\\\n\ \ \ \ \{\{Text\ file\}\ \ \ \ \ \ \ \{.txt\}\ \ \ \ \ \ \ TEXT\}\\\n\ \ \ \ \{\{All\ files\}\ \ \ \ \ \ \ *\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \}\\\n\ \ \ \ \}\]\npPack_read_readme\ \$vPack_readme \
        -text {Load from file} 
    vTcl:DefineAlias "$site_5_0.but109" "Button9" vTcl:WidgetProc "wPack" 1
    entry $site_5_0.ent110 \
        -background white -insertbackground black -textvariable readme \
        -width 40 
    vTcl:DefineAlias "$site_5_0.ent110" "Entry7" vTcl:WidgetProc "wPack" 1
    pack $site_5_0.tex108 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side right 
    pack $site_5_0.but109 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_5_0.ent110 \
        -in $site_5_0 -anchor center -expand 0 -fill none -side left 
    pack $site_4_1.cpd63 \
        -in $site_4_1 -anchor center -expand 0 -fill x -padx 5 -side top 
    pack $site_4_1.cpd64 \
        -in $site_4_1 -anchor center -expand 0 -fill x -padx 5 -side top 
    pack $site_4_1.cpd65 \
        -in $site_4_1 -anchor center -expand 0 -fill x -padx 5 -side top 
    pack $site_4_1.cpd66 \
        -in $site_4_1 -anchor center -expand 0 -fill x -padx 5 -side top 
    pack $site_4_1.cpd67 \
        -in $site_4_1 -anchor center -expand 0 -fill x -padx 5 -side top 
    $top.not56 raise page1
    button $top.cpd62 \
        -command pPack_create -text {Create Atlas Game Pack} 
    vTcl:DefineAlias "$top.cpd62" "Button4" vTcl:WidgetProc "wPack" 1
    ###################
    # SETTING GEOMETRY
    ###################
    pack $top.but57 \
        -in $top -anchor center -expand 0 -fill none -padx 5 -pady 5 \
        -side bottom 
    pack $top.not56 \
        -in $top -anchor center -expand 0 -fill both -side top 
    pack $top.cpd62 \
        -in $top -anchor center -expand 0 -fill x -padx 5 -pady 5 -side top 

    vTcl:FireEvent $base <<Ready>>
}

#############################################################################
## Binding tag:  _TopLevel

bind "_TopLevel" <<Create>> {
    if {![info exists _topcount]} {set _topcount 0}; incr _topcount
}
bind "_TopLevel" <<DeleteWindow>> {
    if {[set ::%W::_modal]} {
                vTcl:Toplevel:WidgetProc %W endmodal
            } else {
                destroy %W; if {$_topcount == 0} {exit}
            }
}
bind "_TopLevel" <Destroy> {
    if {[winfo toplevel %W] == "%W"} {incr _topcount -1}
}
#############################################################################
## Binding tag:  _vTclBalloon


if {![info exists vTcl(sourcing)]} {
bind "_vTclBalloon" <<KillBalloon>> {
    namespace eval ::vTcl::balloon {
        after cancel $id
        if {[winfo exists .vTcl.balloon]} {
            destroy .vTcl.balloon
        }
        set set 0
    }
}
bind "_vTclBalloon" <<vTclBalloon>> {
    if {$::vTcl::balloon::first != 1} {break}

    namespace eval ::vTcl::balloon {
        set first 2
        if {![winfo exists .vTcl]} {
            toplevel .vTcl; wm withdraw .vTcl
        }
        if {![winfo exists .vTcl.balloon]} {
            toplevel .vTcl.balloon -bg black
        }
        wm overrideredirect .vTcl.balloon 1
        label .vTcl.balloon.l  -text ${%W} -relief flat  -bg #ffffaa -fg black -padx 2 -pady 0 -anchor w
        pack .vTcl.balloon.l -side left -padx 1 -pady 1
        wm geometry  .vTcl.balloon  +[expr {[winfo rootx %W]+[winfo width %W]/2}]+[expr {[winfo rooty %W]+[winfo height %W]+4}]
        set set 1
    }
}
bind "_vTclBalloon" <Button> {
    namespace eval ::vTcl::balloon {
        set first 0
    }
    vTcl:FireEvent %W <<KillBalloon>>
}
bind "_vTclBalloon" <Enter> {
    namespace eval ::vTcl::balloon {
        ## self defining balloon?
        if {![info exists %W]} {
            vTcl:FireEvent %W <<SetBalloon>>
        }
        set set 0
        set first 1
        set id [after 500 {vTcl:FireEvent %W <<vTclBalloon>>}]
    }
}
bind "_vTclBalloon" <Leave> {
    namespace eval ::vTcl::balloon {
        set first 0
    }
    vTcl:FireEvent %W <<KillBalloon>>
}
bind "_vTclBalloon" <Motion> {
    namespace eval ::vTcl::balloon {
        if {!$set} {
            after cancel $id
            set id [after 500 {vTcl:FireEvent %W <<vTclBalloon>>}]
        }
    }
}
}

Window show .
Window show .top47
Window show .top61
Window show .top62
Window show .top63

main $argc $argv
