# button.tcl --
#
# This demonstration script creates a toplevel window containing
# several button widgets.
#
# RCS: @(#) $Id: button.tcl,v 1.2 1998/09/14 18:23:27 stanton Exp $

#if {![info exists widgetDemo]} {
#    error "This script should be run from the \"widget\" demo."
#}


# positionWindow --
# This procedure is invoked by most of the demos to position a
# new demo window.
#
# Arguments:
# w -		The name of the window to position.

proc positionWindow w {
    wm geometry $w +300+300
}


proc showCode w {
    global tk_library
    set file [string range $w 1 end].tcl
    if ![winfo exists .code] {
	toplevel .code
	frame .code.buttons
	pack .code.buttons -side bottom -fill x
	button .code.buttons.dismiss -text Dismiss \
            -default active -command "destroy .code"
	button .code.buttons.rerun -text "Rerun Demo" -command {
	    eval [.code.text get 1.0 end]
	}
	pack .code.buttons.dismiss .code.buttons.rerun -side left \
	    -expand 1 -pady 2
	frame .code.frame
	pack  .code.frame -expand yes -fill both -padx 1 -pady 1
	text .code.text -height 40 -wrap word\
	    -xscrollcommand ".code.xscroll set" \
	    -yscrollcommand ".code.yscroll set" \
	    -setgrid 1 -highlightthickness 0 -pady 2 -padx 3
	scrollbar .code.xscroll -command ".code.text xview" \
	    -highlightthickness 0 -orient horizontal
	scrollbar .code.yscroll -command ".code.text yview" \
	    -highlightthickness 0 -orient vertical

	grid .code.text -in .code.frame -padx 1 -pady 1 \
	    -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
	grid .code.yscroll -in .code.frame -padx 1 -pady 1 \
	    -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
#	grid .code.xscroll -in .code.frame -padx 1 -pady 1 \
#	    -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
	grid rowconfig    .code.frame 0 -weight 1 -minsize 0
	grid columnconfig .code.frame 0 -weight 1 -minsize 0
    } else {
	wm deiconify .code
	raise .code
    }
    wm title .code "Demo code: [file join $tk_library demos $file]"
    wm iconname .code $file
    set id [open [file join $tk_library demos $file]]
    .code.text delete 1.0 end
    .code.text insert 1.0 [read $id]
    .code.text mark set insert 1.0
    close $id
}


array set widgetFont {
    main   {Helvetica 12}
    bold   {Helvetica 12 bold}
    title  {Helvetica 18 bold}
    status {Helvetica 10}
    vars   {Helvetica 14}
}

set font $widgetFont(main)
set borderTop 20
set screenWidth 1024
set screenHeight 768
set screenWidthHalf [expr $screenWidth / 2]
set screenHeightHalf [expr [expr $screenHeight - $borderTop] / 2]



proc selectDir {dir win} {
	set mydir [tk_chooseDirectory -initialdir $dir -parent $win -title "Choose directory:" -mustexist 1]
	if {[string length $mydir] != 0} {
		return $mydir
	} else {
		return $dir	
	}
}




# Create Multi Game Saturn Disc
proc showCMGSD {} {
	global screenWidthHalf screenHeightHalf borderTop
	catch {destroy .cMGSD}
	toplevel .cMGSD
	wm title .cMGSD "Create Multi Game Saturn Disc"
	wm iconname .cMGSD "Multi Game"
	wm geometry .cMGSD =[expr $screenWidthHalf]x[expr $screenHeightHalf]+[expr $screenWidthHalf]+[expr $borderTop]
}


# Create Single Game Saturn Disc
proc showCSGSD {} {
	global screenWidthHalf screenHeightHalf borderTop
	catch {destroy .cSGSD}
	toplevel .cSGSD
	wm title .cSGSD "Create Single Game Saturn Disc"
	wm iconname .cSGSD "Single Game"
	wm geometry .cSGSD =[expr $screenWidthHalf]x[expr $screenHeightHalf]+[expr $screenWidthHalf]+[expr $borderTop + $screenHeightHalf]
}

# Create Atlas Game Pack
proc showCAGP {} {
	global screenWidthHalf screenHeightHalf borderTop font tk_library
	set dir_cd [file join $tk_library demos images]
#"."
	catch {destroy .cAGP}
	toplevel .cAGP
	wm title .cAGP "Create Atlas Game Pack"
	wm iconname .cAGP "AGP"
	wm geometry .cAGP =[expr $screenWidthHalf]x[expr $screenHeightHalf]+0+[expr $borderTop + $screenHeightHalf]
# Required stuff
	labelframe .cAGP.req -text "Required" -padx 2 -pady 2
	pack .cAGP.req -side top -expand 0
#	data folder
	labelframe .cAGP.req.cd -text "CD root directory"
	pack .cAGP.req.cd -side top
	label .cAGP.req.cd.l -font $font -wraplength 4i -justify left -text "Specify the directory which shall become the CD root. It must include the binary executable.\nIt's name must not exceed 8 characters."
	pack .cAGP.req.cd.l -side left
#	grid .cAGP.req.cd.l -row 0 -column 0 -pady 2m -padx 2m

	entry .cAGP.req.cd.e -width 30 -textvariable dir_cd
	button .cAGP.req.cd.b -pady 0 -padx 2m -text "Select Dir." -command "set dir_cd [selectDir $dir_cd .cAGP]"
#	bind $w.dir.e <Return> "loadDir $w"
	pack .cAGP.req.cd.e -side left
#-fill both -padx 2m     -pady 2m -expand true
	pack .cAGP.req.cd.b -side right 
#left  -fill y  -padx {0 2m}  -pady 2m


#	button .cAGP.req.d.code -text "See Code" -width 15
#	pack .cAGP.req.d.code -side right -expand 1
#	grid .cAGP.req.d.code -row 0 -column 1 -pady 2m -padx 2m
# Optional stuff
	labelframe .cAGP.o -text "Optional" -padx 2 -pady 2
	pack .cAGP.o -side top -expand 0
	button .cAGP.o.code -text "See Code" -width 15
	pack .cAGP.o.code -side top -expand 1
}




set w .button
catch {destroy $w}
toplevel $w
wm title $w "Atlas Creator"
wm iconname $w "button"
wm geometry $w =[expr $screenWidthHalf]x[expr $screenHeightHalf]+0+[expr $borderTop]




label $w.msg -font $font -wraplength 4i -justify left -text "Atlas:\nAtlas is a multi game boot disc application for SEGA Saturn. With Atlas, you can put up to 50 games on one Saturn disc.\n\nAtlas Game Packs:\nIn Atlas, each game can present additional information like screenshots, control info, short text description or even audio information. All this content is included in a single file called Atlas Game Pack. \n\nAtlas Creator:\nThis application, the Atlas Maker, allows to do both, to create Saturn discs from up to 50 Atlas Game Packs as well as to create Atlas Game Packs.\n"
pack $w.msg -side top

frame $w.buttons
pack $w.buttons -side bottom -fill x -pady 2m
button $w.buttons.dismiss -text "Close all" -command "catch {destroy .cMGSD};catch {destroy .cSGSD};catch {destroy .cAGP};catch {destroy $w}"
button $w.buttons.code -text "See Code" -command "showCode $w"
pack $w.buttons.dismiss $w.buttons.code -side left -expand 1

button $w.b1 -text "Create Multi Game Saturn Disc" -width 25 \
    -command "showCMGSD"
button $w.b2 -text "Create Single Game Saturn Disc" -width 25 \
    -command "showCSGSD"
button $w.b3 -text "Create Atlas Game Pack" -width 25 \
    -command "showCAGP"
pack $w.b1 $w.b2 $w.b3 -side top -expand yes -pady 2
